/*
 * Performance Metrics Name Space Map
 * Built by pmgenmap.sh from the file
 * hostmetrics.map
 * on Thu Jul 30 14:21:13 UTC 2026
 *
 * Do not edit this file!
 */

const char *hostmetrics[] = {
#define NRCPUS	0
	"hinv.ncpu",
#define NRDISK	1
	"hinv.ndisk",
#define NRNUMA	2
	"hinv.nnode",
#define NRINTF	3
	"hinv.ninterface",
#define NRGPUS	4
	"nvidia.numcards",
#define MAP_DMNAME	5
	"hinv.map.dmname",
#define MAP_MDNAME	6
	"hinv.map.mdname",

#define HOST_PAGESIZE	7
	"hinv.pagesize",
#define HOST_HERTZ	8
	"kernel.all.hz",
#define HOST_PID_MAX	9
	"kernel.all.pid_max",
#define HOST_BTIME	10
	"kernel.all.boottime",

#define HOST_RELEASE	11
	"kernel.uname.release",
#define HOST_VERSION	12
	"kernel.uname.version",
#define HOST_MACHINE	13
	"kernel.uname.machine",
#define HOST_NODENAME	14
	"kernel.uname.nodename",

/* end marker - provides count */
#define HOST_NMETRICS	15
	".",

};

