/********************************************************************************
** Form generated from reading UI file 'recorddialog.ui'
**
** Created by: Qt User Interface Compiler version 5.15.19
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_RECORDDIALOG_H
#define UI_RECORDDIALOG_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_RecordDialog
{
public:
    QGridLayout *gridLayout;
    QVBoxLayout *vboxLayout;
    QHBoxLayout *hboxLayout;
    QVBoxLayout *vboxLayout1;
    QRadioButton *allGadgetsRadioButton;
    QRadioButton *selectedRadioButton;
    QSpacerItem *spacerItem;
    QVBoxLayout *vboxLayout2;
    QLabel *deltaLabel;
    QHBoxLayout *hboxLayout1;
    QLineEdit *deltaLineEdit;
    QComboBox *deltaUnitsComboBox;
    QVBoxLayout *vboxLayout3;
    QHBoxLayout *hboxLayout2;
    QLabel *folioTextLabel;
    QLineEdit *folioLineEdit;
    QPushButton *folioPushButton;
    QHBoxLayout *hboxLayout3;
    QLabel *viewTextLabel;
    QLineEdit *viewLineEdit;
    QPushButton *viewPushButton;
    QHBoxLayout *hboxLayout4;
    QLabel *archiveTextLabel;
    QLineEdit *archiveLineEdit;
    QPushButton *archivePushButton;
    QSpacerItem *spacerItem1;
    QHBoxLayout *hboxLayout5;
    QSpacerItem *spacerItem2;
    QPushButton *buttonOk;
    QPushButton *buttonCancel;

    void setupUi(QDialog *RecordDialog)
    {
        if (RecordDialog->objectName().isEmpty())
            RecordDialog->setObjectName(QString::fromUtf8("RecordDialog"));
        RecordDialog->setWindowModality(Qt::WindowModal);
        RecordDialog->resize(410, 240);
        QSizePolicy sizePolicy(QSizePolicy::Preferred, QSizePolicy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(RecordDialog->sizePolicy().hasHeightForWidth());
        RecordDialog->setSizePolicy(sizePolicy);
        RecordDialog->setMinimumSize(QSize(410, 240));
        RecordDialog->setMaximumSize(QSize(16777215, 240));
        const QIcon icon = QIcon(QString::fromUtf8(":/images/archive.png"));
        RecordDialog->setWindowIcon(icon);
        RecordDialog->setSizeGripEnabled(false);
        gridLayout = new QGridLayout(RecordDialog);
#ifndef Q_OS_MAC
        gridLayout->setSpacing(6);
#endif
#ifndef Q_OS_MAC
        gridLayout->setContentsMargins(9, 9, 9, 9);
#endif
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        vboxLayout = new QVBoxLayout();
#ifndef Q_OS_MAC
        vboxLayout->setSpacing(6);
#endif
#ifndef Q_OS_MAC
        vboxLayout->setContentsMargins(0, 0, 0, 0);
#endif
        vboxLayout->setObjectName(QString::fromUtf8("vboxLayout"));
        hboxLayout = new QHBoxLayout();
        hboxLayout->setSpacing(0);
#ifndef Q_OS_MAC
        hboxLayout->setContentsMargins(0, 0, 0, 0);
#endif
        hboxLayout->setObjectName(QString::fromUtf8("hboxLayout"));
        vboxLayout1 = new QVBoxLayout();
#ifndef Q_OS_MAC
        vboxLayout1->setSpacing(6);
#endif
#ifndef Q_OS_MAC
        vboxLayout1->setContentsMargins(0, 0, 0, 0);
#endif
        vboxLayout1->setObjectName(QString::fromUtf8("vboxLayout1"));
        allGadgetsRadioButton = new QRadioButton(RecordDialog);
        allGadgetsRadioButton->setObjectName(QString::fromUtf8("allGadgetsRadioButton"));
        QSizePolicy sizePolicy1(QSizePolicy::Minimum, QSizePolicy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(allGadgetsRadioButton->sizePolicy().hasHeightForWidth());
        allGadgetsRadioButton->setSizePolicy(sizePolicy1);
        allGadgetsRadioButton->setChecked(true);

        vboxLayout1->addWidget(allGadgetsRadioButton);

        selectedRadioButton = new QRadioButton(RecordDialog);
        selectedRadioButton->setObjectName(QString::fromUtf8("selectedRadioButton"));
        selectedRadioButton->setChecked(false);

        vboxLayout1->addWidget(selectedRadioButton);


        hboxLayout->addLayout(vboxLayout1);

        spacerItem = new QSpacerItem(20, 20, QSizePolicy::Fixed, QSizePolicy::Minimum);

        hboxLayout->addItem(spacerItem);

        vboxLayout2 = new QVBoxLayout();
#ifndef Q_OS_MAC
        vboxLayout2->setSpacing(6);
#endif
        vboxLayout2->setContentsMargins(0, 0, 0, 0);
        vboxLayout2->setObjectName(QString::fromUtf8("vboxLayout2"));
        deltaLabel = new QLabel(RecordDialog);
        deltaLabel->setObjectName(QString::fromUtf8("deltaLabel"));
        deltaLabel->setAlignment(Qt::AlignVCenter);
        deltaLabel->setWordWrap(false);

        vboxLayout2->addWidget(deltaLabel);

        hboxLayout1 = new QHBoxLayout();
#ifndef Q_OS_MAC
        hboxLayout1->setSpacing(6);
#endif
        hboxLayout1->setContentsMargins(0, 0, 0, 0);
        hboxLayout1->setObjectName(QString::fromUtf8("hboxLayout1"));
        deltaLineEdit = new QLineEdit(RecordDialog);
        deltaLineEdit->setObjectName(QString::fromUtf8("deltaLineEdit"));
        deltaLineEdit->setAlignment(Qt::AlignRight);

        hboxLayout1->addWidget(deltaLineEdit);

        deltaUnitsComboBox = new QComboBox(RecordDialog);
        deltaUnitsComboBox->addItem(QString());
        deltaUnitsComboBox->addItem(QString());
        deltaUnitsComboBox->addItem(QString());
        deltaUnitsComboBox->addItem(QString());
        deltaUnitsComboBox->addItem(QString());
        deltaUnitsComboBox->addItem(QString());
        deltaUnitsComboBox->setObjectName(QString::fromUtf8("deltaUnitsComboBox"));
        sizePolicy1.setHeightForWidth(deltaUnitsComboBox->sizePolicy().hasHeightForWidth());
        deltaUnitsComboBox->setSizePolicy(sizePolicy1);
        deltaUnitsComboBox->setMaxCount(6);
        deltaUnitsComboBox->setInsertPolicy(QComboBox::NoInsert);
        deltaUnitsComboBox->setDuplicatesEnabled(false);

        hboxLayout1->addWidget(deltaUnitsComboBox);


        vboxLayout2->addLayout(hboxLayout1);


        hboxLayout->addLayout(vboxLayout2);


        vboxLayout->addLayout(hboxLayout);

        vboxLayout3 = new QVBoxLayout();
#ifndef Q_OS_MAC
        vboxLayout3->setSpacing(6);
#endif
        vboxLayout3->setContentsMargins(0, 0, 0, 0);
        vboxLayout3->setObjectName(QString::fromUtf8("vboxLayout3"));
        hboxLayout2 = new QHBoxLayout();
#ifndef Q_OS_MAC
        hboxLayout2->setSpacing(6);
#endif
#ifndef Q_OS_MAC
        hboxLayout2->setContentsMargins(0, 0, 0, 0);
#endif
        hboxLayout2->setObjectName(QString::fromUtf8("hboxLayout2"));
        folioTextLabel = new QLabel(RecordDialog);
        folioTextLabel->setObjectName(QString::fromUtf8("folioTextLabel"));
        folioTextLabel->setWordWrap(false);

        hboxLayout2->addWidget(folioTextLabel);

        folioLineEdit = new QLineEdit(RecordDialog);
        folioLineEdit->setObjectName(QString::fromUtf8("folioLineEdit"));

        hboxLayout2->addWidget(folioLineEdit);

        folioPushButton = new QPushButton(RecordDialog);
        folioPushButton->setObjectName(QString::fromUtf8("folioPushButton"));
        const QIcon icon1 = QIcon(QString::fromUtf8(":/images/folio.png"));
        folioPushButton->setIcon(icon1);

        hboxLayout2->addWidget(folioPushButton);


        vboxLayout3->addLayout(hboxLayout2);

        hboxLayout3 = new QHBoxLayout();
#ifndef Q_OS_MAC
        hboxLayout3->setSpacing(6);
#endif
        hboxLayout3->setContentsMargins(0, 0, 0, 0);
        hboxLayout3->setObjectName(QString::fromUtf8("hboxLayout3"));
        viewTextLabel = new QLabel(RecordDialog);
        viewTextLabel->setObjectName(QString::fromUtf8("viewTextLabel"));
        viewTextLabel->setWordWrap(false);

        hboxLayout3->addWidget(viewTextLabel);

        viewLineEdit = new QLineEdit(RecordDialog);
        viewLineEdit->setObjectName(QString::fromUtf8("viewLineEdit"));

        hboxLayout3->addWidget(viewLineEdit);

        viewPushButton = new QPushButton(RecordDialog);
        viewPushButton->setObjectName(QString::fromUtf8("viewPushButton"));
        const QIcon icon2 = QIcon(QString::fromUtf8(":/images/view.png"));
        viewPushButton->setIcon(icon2);

        hboxLayout3->addWidget(viewPushButton);


        vboxLayout3->addLayout(hboxLayout3);

        hboxLayout4 = new QHBoxLayout();
#ifndef Q_OS_MAC
        hboxLayout4->setSpacing(6);
#endif
        hboxLayout4->setContentsMargins(0, 0, 0, 0);
        hboxLayout4->setObjectName(QString::fromUtf8("hboxLayout4"));
        archiveTextLabel = new QLabel(RecordDialog);
        archiveTextLabel->setObjectName(QString::fromUtf8("archiveTextLabel"));
        archiveTextLabel->setWordWrap(false);

        hboxLayout4->addWidget(archiveTextLabel);

        archiveLineEdit = new QLineEdit(RecordDialog);
        archiveLineEdit->setObjectName(QString::fromUtf8("archiveLineEdit"));

        hboxLayout4->addWidget(archiveLineEdit);

        archivePushButton = new QPushButton(RecordDialog);
        archivePushButton->setObjectName(QString::fromUtf8("archivePushButton"));
        archivePushButton->setIcon(icon);

        hboxLayout4->addWidget(archivePushButton);


        vboxLayout3->addLayout(hboxLayout4);


        vboxLayout->addLayout(vboxLayout3);

        spacerItem1 = new QSpacerItem(20, 5, QSizePolicy::Minimum, QSizePolicy::Fixed);

        vboxLayout->addItem(spacerItem1);

        hboxLayout5 = new QHBoxLayout();
#ifndef Q_OS_MAC
        hboxLayout5->setSpacing(6);
#endif
        hboxLayout5->setContentsMargins(0, 0, 0, 0);
        hboxLayout5->setObjectName(QString::fromUtf8("hboxLayout5"));
        spacerItem2 = new QSpacerItem(20, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        hboxLayout5->addItem(spacerItem2);

        buttonOk = new QPushButton(RecordDialog);
        buttonOk->setObjectName(QString::fromUtf8("buttonOk"));
        buttonOk->setAutoDefault(true);

        hboxLayout5->addWidget(buttonOk);

        buttonCancel = new QPushButton(RecordDialog);
        buttonCancel->setObjectName(QString::fromUtf8("buttonCancel"));
        buttonCancel->setAutoDefault(true);

        hboxLayout5->addWidget(buttonCancel);


        vboxLayout->addLayout(hboxLayout5);


        gridLayout->addLayout(vboxLayout, 0, 0, 1, 1);


        retranslateUi(RecordDialog);

        deltaUnitsComboBox->setCurrentIndex(1);
        buttonOk->setDefault(true);


        QMetaObject::connectSlotsByName(RecordDialog);
    } // setupUi

    void retranslateUi(QDialog *RecordDialog)
    {
        RecordDialog->setWindowTitle(QCoreApplication::translate("RecordDialog", "Record", nullptr));
#if QT_CONFIG(tooltip)
        RecordDialog->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
        allGadgetsRadioButton->setText(QCoreApplication::translate("RecordDialog", "All Charts in Tab", nullptr));
        selectedRadioButton->setText(QCoreApplication::translate("RecordDialog", "Selected Chart Only", nullptr));
        deltaLabel->setText(QCoreApplication::translate("RecordDialog", "Logging Interval:", nullptr));
        deltaLineEdit->setText(QString());
        deltaUnitsComboBox->setItemText(0, QCoreApplication::translate("RecordDialog", "Milliseconds", nullptr));
        deltaUnitsComboBox->setItemText(1, QCoreApplication::translate("RecordDialog", "Seconds", nullptr));
        deltaUnitsComboBox->setItemText(2, QCoreApplication::translate("RecordDialog", "Minutes", nullptr));
        deltaUnitsComboBox->setItemText(3, QCoreApplication::translate("RecordDialog", "Hours", nullptr));
        deltaUnitsComboBox->setItemText(4, QCoreApplication::translate("RecordDialog", "Days", nullptr));
        deltaUnitsComboBox->setItemText(5, QCoreApplication::translate("RecordDialog", "Weeks", nullptr));

        folioTextLabel->setText(QCoreApplication::translate("RecordDialog", "Folio", nullptr));
        folioPushButton->setText(QCoreApplication::translate("RecordDialog", "...", nullptr));
        viewTextLabel->setText(QCoreApplication::translate("RecordDialog", "View", nullptr));
        viewPushButton->setText(QCoreApplication::translate("RecordDialog", "...", nullptr));
        archiveTextLabel->setText(QCoreApplication::translate("RecordDialog", "Archive", nullptr));
        archivePushButton->setText(QCoreApplication::translate("RecordDialog", "...", nullptr));
        buttonOk->setText(QCoreApplication::translate("RecordDialog", "&OK", nullptr));
#if QT_CONFIG(shortcut)
        buttonOk->setShortcut(QString());
#endif // QT_CONFIG(shortcut)
        buttonCancel->setText(QCoreApplication::translate("RecordDialog", "&Cancel", nullptr));
#if QT_CONFIG(shortcut)
        buttonCancel->setShortcut(QString());
#endif // QT_CONFIG(shortcut)
    } // retranslateUi

};

namespace Ui {
    class RecordDialog: public Ui_RecordDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_RECORDDIALOG_H
