/********************************************************************************
** Form generated from reading UI file 'searchdialog.ui'
**
** Created by: Qt User Interface Compiler version 5.15.19
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SEARCHDIALOG_H
#define UI_SEARCHDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>

QT_BEGIN_NAMESPACE

class Ui_SearchDialog
{
public:
    QGridLayout *gridLayout;
    QGridLayout *gridLayout1;
    QLabel *hostLabel;
    QLabel *metricLabel;
    QLabel *instanceLabel;
    QLineEdit *hostPattern;
    QLineEdit *metricPattern;
    QLineEdit *instancePattern;
    QHBoxLayout *hboxLayout;
    QPushButton *buttonAll;
    QLabel *resultStatus;
    QHBoxLayout *hboxLayout1;
    QSpacerItem *spacerItem;
    QPushButton *buttonSearch;
    QPushButton *buttonClear;
    QListWidget *matchList;
    QHBoxLayout *hboxLayout2;
    QSpacerItem *spacerItem1;
    QPushButton *buttonOk;
    QPushButton *buttonCancel;

    void setupUi(QDialog *SearchDialog)
    {
        if (SearchDialog->objectName().isEmpty())
            SearchDialog->setObjectName(QString::fromUtf8("SearchDialog"));
        SearchDialog->setWindowModality(Qt::WindowModal);
        SearchDialog->resize(452, 345);
        QSizePolicy sizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(SearchDialog->sizePolicy().hasHeightForWidth());
        SearchDialog->setSizePolicy(sizePolicy);
        gridLayout = new QGridLayout(SearchDialog);
#ifndef Q_OS_MAC
        gridLayout->setSpacing(6);
#endif
#ifndef Q_OS_MAC
        gridLayout->setContentsMargins(9, 9, 9, 9);
#endif
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        gridLayout1 = new QGridLayout();
        gridLayout1->setSpacing(0);
#ifndef Q_OS_MAC
        gridLayout1->setContentsMargins(0, 0, 0, 0);
#endif
        gridLayout1->setObjectName(QString::fromUtf8("gridLayout1"));
        hostLabel = new QLabel(SearchDialog);
        hostLabel->setObjectName(QString::fromUtf8("hostLabel"));

        gridLayout1->addWidget(hostLabel, 0, 0, 1, 1);

        metricLabel = new QLabel(SearchDialog);
        metricLabel->setObjectName(QString::fromUtf8("metricLabel"));

        gridLayout1->addWidget(metricLabel, 1, 0, 1, 1);

        instanceLabel = new QLabel(SearchDialog);
        instanceLabel->setObjectName(QString::fromUtf8("instanceLabel"));

        gridLayout1->addWidget(instanceLabel, 2, 0, 1, 1);

        hostPattern = new QLineEdit(SearchDialog);
        hostPattern->setObjectName(QString::fromUtf8("hostPattern"));

        gridLayout1->addWidget(hostPattern, 0, 1, 1, 1);

        metricPattern = new QLineEdit(SearchDialog);
        metricPattern->setObjectName(QString::fromUtf8("metricPattern"));

        gridLayout1->addWidget(metricPattern, 1, 1, 1, 1);

        instancePattern = new QLineEdit(SearchDialog);
        instancePattern->setObjectName(QString::fromUtf8("instancePattern"));

        gridLayout1->addWidget(instancePattern, 2, 1, 1, 1);


        gridLayout->addLayout(gridLayout1, 0, 0, 1, 1);

        hboxLayout = new QHBoxLayout();
#ifndef Q_OS_MAC
        hboxLayout->setSpacing(6);
#endif
        hboxLayout->setContentsMargins(0, 0, 0, 0);
        hboxLayout->setObjectName(QString::fromUtf8("hboxLayout"));
        buttonAll = new QPushButton(SearchDialog);
        buttonAll->setObjectName(QString::fromUtf8("buttonAll"));
        sizePolicy.setHeightForWidth(buttonAll->sizePolicy().hasHeightForWidth());
        buttonAll->setSizePolicy(sizePolicy);

        hboxLayout->addWidget(buttonAll);

        resultStatus = new QLabel(SearchDialog);
        resultStatus->setObjectName(QString::fromUtf8("resultStatus"));

        hboxLayout->addWidget(resultStatus);


        gridLayout->addLayout(hboxLayout, 2, 0, 1, 1);

        hboxLayout1 = new QHBoxLayout();
#ifndef Q_OS_MAC
        hboxLayout1->setSpacing(6);
#endif
        hboxLayout1->setContentsMargins(0, 0, 0, 0);
        hboxLayout1->setObjectName(QString::fromUtf8("hboxLayout1"));
        spacerItem = new QSpacerItem(271, 27, QSizePolicy::Expanding, QSizePolicy::Minimum);

        hboxLayout1->addItem(spacerItem);

        buttonSearch = new QPushButton(SearchDialog);
        buttonSearch->setObjectName(QString::fromUtf8("buttonSearch"));

        hboxLayout1->addWidget(buttonSearch);

        buttonClear = new QPushButton(SearchDialog);
        buttonClear->setObjectName(QString::fromUtf8("buttonClear"));
        sizePolicy.setHeightForWidth(buttonClear->sizePolicy().hasHeightForWidth());
        buttonClear->setSizePolicy(sizePolicy);

        hboxLayout1->addWidget(buttonClear);


        gridLayout->addLayout(hboxLayout1, 1, 0, 1, 1);

        matchList = new QListWidget(SearchDialog);
        matchList->setObjectName(QString::fromUtf8("matchList"));
        matchList->setSelectionMode(QAbstractItemView::ExtendedSelection);

        gridLayout->addWidget(matchList, 3, 0, 1, 1);

        hboxLayout2 = new QHBoxLayout();
#ifndef Q_OS_MAC
        hboxLayout2->setSpacing(6);
#endif
        hboxLayout2->setContentsMargins(0, 0, 0, 0);
        hboxLayout2->setObjectName(QString::fromUtf8("hboxLayout2"));
        spacerItem1 = new QSpacerItem(71, 27, QSizePolicy::Expanding, QSizePolicy::Minimum);

        hboxLayout2->addItem(spacerItem1);

        buttonOk = new QPushButton(SearchDialog);
        buttonOk->setObjectName(QString::fromUtf8("buttonOk"));

        hboxLayout2->addWidget(buttonOk);

        buttonCancel = new QPushButton(SearchDialog);
        buttonCancel->setObjectName(QString::fromUtf8("buttonCancel"));

        hboxLayout2->addWidget(buttonCancel);


        gridLayout->addLayout(hboxLayout2, 4, 0, 1, 1);

        QWidget::setTabOrder(hostPattern, metricPattern);
        QWidget::setTabOrder(metricPattern, instancePattern);
        QWidget::setTabOrder(instancePattern, buttonSearch);
        QWidget::setTabOrder(buttonSearch, matchList);
        QWidget::setTabOrder(matchList, buttonOk);
        QWidget::setTabOrder(buttonOk, buttonCancel);
        QWidget::setTabOrder(buttonCancel, buttonClear);

        retranslateUi(SearchDialog);

        QMetaObject::connectSlotsByName(SearchDialog);
    } // setupUi

    void retranslateUi(QDialog *SearchDialog)
    {
        SearchDialog->setWindowTitle(QCoreApplication::translate("SearchDialog", "Metric Search", nullptr));
        hostLabel->setText(QCoreApplication::translate("SearchDialog", "Host Name Pattern:", nullptr));
        metricLabel->setText(QCoreApplication::translate("SearchDialog", "Metric Name Pattern:", nullptr));
        instanceLabel->setText(QCoreApplication::translate("SearchDialog", "Instance Name Pattern:", nullptr));
        buttonAll->setText(QCoreApplication::translate("SearchDialog", "Select &All", nullptr));
        resultStatus->setText(QString());
        buttonSearch->setText(QCoreApplication::translate("SearchDialog", "&Search", nullptr));
        buttonClear->setText(QCoreApplication::translate("SearchDialog", "C&lear", nullptr));
        buttonOk->setText(QCoreApplication::translate("SearchDialog", "&OK", nullptr));
        buttonCancel->setText(QCoreApplication::translate("SearchDialog", "&Cancel", nullptr));
    } // retranslateUi

};

namespace Ui {
    class SearchDialog: public Ui_SearchDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_SEARCHDIALOG_H
