/*
 * Performance Metrics Name Space Map
 * Built by pmgenmap.sh from the file
 * procmetrics.map
 * on Thu Jul 30 14:21:13 UTC 2026
 *
 * Do not edit this file!
 */

/*
 *
 * metric names and access macros for photoproc.[ch]
 * use the hotproc variants here, and at runtime we
 * switch metrics (proc.* or hotproc.*) via pointer
 * manipulation.
 *
 */
const char *procmetrics[] = {
/* /proc/pid/stat */
#define TASK_GEN_NAME	0
	"hotproc.psinfo.cmd",
#define TASK_GEN_STATE	1
	"hotproc.psinfo.sname",
#define TASK_GEN_PPID	2
	"hotproc.psinfo.ppid",
#define TASK_MEM_MINFLT	3
	"hotproc.psinfo.minflt",
#define TASK_MEM_MAJFLT	4
	"hotproc.psinfo.maj_flt",
#define TASK_CPU_UTIME	5
	"hotproc.psinfo.utime",
#define TASK_CPU_STIME	6
	"hotproc.psinfo.stime",
#define TASK_CPU_PRIO	7
	"hotproc.psinfo.priority",
#define TASK_CPU_NICE	8
	"hotproc.psinfo.nice",
#define TASK_GEN_BTIME	9
	"hotproc.psinfo.start_time",
#define TASK_MEM_VMEM	10
	"hotproc.psinfo.vsize",
#define TASK_MEM_RMEM	11
	"hotproc.psinfo.rss",
#define TASK_CPU_CURCPU	12
	"hotproc.psinfo.processor",
#define TASK_CPU_RTPRIO	13
	"hotproc.psinfo.rt_priority",
#define TASK_CPU_POLICY	14
	"hotproc.psinfo.policy",
#define TASK_CPU_BLKDELAY	15
	"hotproc.psinfo.delayacct_blkio_time",
#define TASK_GEN_WCHAN	16
	"hotproc.psinfo.wchan_s",
#define TASK_CPU_NVCTXSW	17
	"hotproc.psinfo.vctxsw",
#define TASK_CPU_NIVCTXSW	18
	"hotproc.psinfo.nvctxsw",

/* /proc/pid/status */
#define TASK_GEN_NTHR	19
	"hotproc.psinfo.threads",
#define TASK_GEN_TGID	20
	"hotproc.psinfo.tgid",
#define TASK_GEN_PID	21
	"hotproc.psinfo.pid",
#define TASK_GEN_ENVID	22
	"hotproc.namespaces.envid",
#define TASK_GEN_VPID	23
	"hotproc.namespaces.vpid",
#define TASK_GEN_RUID	24
	"hotproc.id.uid",
#define TASK_GEN_EUID	25
	"hotproc.id.euid",
#define TASK_GEN_SUID	26
	"hotproc.id.suid",
#define TASK_GEN_FSUID	27
	"hotproc.id.fsuid",
#define TASK_GEN_RGID	28
	"hotproc.id.gid",
#define TASK_GEN_EGID	29
	"hotproc.id.egid",
#define TASK_GEN_SGID	30
	"hotproc.id.sgid",
#define TASK_GEN_FSGID	31
	"hotproc.id.fsgid",
#define TASK_MEM_VDATA	32
	"hotproc.memory.vmdata",
#define TASK_MEM_VSTACK	33
	"hotproc.memory.vmstack",
#define TASK_MEM_VEXEC	34
	"hotproc.memory.vmexe",
#define TASK_MEM_VLIBS	35
	"hotproc.memory.vmlib",
#define TASK_MEM_VSWAP	36
	"hotproc.memory.vmswap",
#define TASK_MEM_VLOCK	37
	"hotproc.memory.vmlock",

/* /proc/pid/io */
#define TASK_DSK_RSZ	38
	"hotproc.io.read_bytes",
#define TASK_DSK_WSZ	39
	"hotproc.io.write_bytes",
#define TASK_DSK_CWSZ	40
	"hotproc.io.cancelled_write_bytes",

/* /proc/pid/schedstat */
#define TASK_CPU_RUNDELAY	41
	"hotproc.schedstat.run_delay",

/* /proc/pid/smaps_rollup */
#define TASK_MEM_PMEM	42
	"hotproc.smaps.pss",

/* /proc/pid/environ */
#define TASK_GEN_ENVIRON	43
	"hotproc.psinfo.environ",

/* getpwuid */
#define TASK_GEN_RUIDNM	44
	"hotproc.id.uid_nm",
#define TASK_GEN_EUIDNM	45
	"hotproc.id.euid_nm",
#define TASK_GEN_SUIDNM	46
	"hotproc.id.suid_nm",
#define TASK_GEN_FSUIDNM	47
	"hotproc.id.fsuid_nm",

/* getgruid */
#define TASK_GEN_RGIDNM	48
	"hotproc.id.gid_nm",
#define TASK_GEN_EGIDNM	49
	"hotproc.id.egid_nm",
#define TASK_GEN_SGIDNM	50
	"hotproc.id.sgid_nm",
#define TASK_GEN_FSGIDNM	51
	"hotproc.id.fsgid_nm",

/* /proc/pid/cgroup */
#define TASK_GEN_CONTAINER	52
	"hotproc.id.container",
#define TASK_GEN_CGROUP	53
	"hotproc.psinfo.cgroups",

/* /proc/pid/cmdline */
#define TASK_GEN_CMDLINE	54
	"hotproc.psinfo.psargs",

/* end marker - provides count */
#define TASK_NMETRICS	55
	".",

};

