/********************************************************************************
** Form generated from reading UI file 'chartdialog.ui'
**
** Created by: Qt User Interface Compiler version 5.15.19
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CHARTDIALOG_H
#define UI_CHARTDIALOG_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QTreeWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "chart.h"
#include "namespace.h"
#include "qed_colorpicker.h"

QT_BEGIN_NAMESPACE

class Ui_ChartDialog
{
public:
    QGridLayout *gridLayout;
    QTabWidget *tabWidget;
    QWidget *Widget2;
    QGridLayout *gridLayout1;
    QVBoxLayout *vboxLayout;
    QSpacerItem *spacerItem;
    QGridLayout *gridLayout2;
    QComboBox *colorSchemeComboBox;
    QComboBox *typeComboBox;
    QLabel *colorSchemeLabel;
    QHBoxLayout *hboxLayout;
    QSpacerItem *spacerItem1;
    QRadioButton *legendOn;
    QSpacerItem *spacerItem2;
    QRadioButton *legendOff;
    QSpacerItem *spacerItem3;
    QLabel *legendTextLabel;
    QLabel *antiAliasingTextLabel;
    QHBoxLayout *hboxLayout1;
    QSpacerItem *spacerItem4;
    QRadioButton *antiAliasingAuto;
    QSpacerItem *spacerItem5;
    QRadioButton *antiAliasingOn;
    QSpacerItem *spacerItem6;
    QRadioButton *antiAliasingOff;
    QSpacerItem *spacerItem7;
    QLineEdit *titleLineEdit;
    QLabel *titleTextLabel;
    QLabel *styleLabel;
    QSpacerItem *spacerItem8;
    QGroupBox *yAxisGroupBox;
    QGridLayout *gridLayout3;
    QGridLayout *gridLayout4;
    QLabel *maxTextLabel;
    QHBoxLayout *hboxLayout2;
    QDoubleSpinBox *yAxisMinimum;
    QSpacerItem *spacerItem9;
    QLabel *minTextLabel;
    QHBoxLayout *hboxLayout3;
    QSpacerItem *spacerItem10;
    QRadioButton *autoScaleOn;
    QSpacerItem *spacerItem11;
    QRadioButton *autoScaleOff;
    QSpacerItem *spacerItem12;
    QHBoxLayout *hboxLayout4;
    QDoubleSpinBox *yAxisMaximum;
    QSpacerItem *spacerItem13;
    QLabel *autoScale;
    QCheckBox *rateConvertCheckBox;
    QSpacerItem *spacerItem14;
    QWidget *Widget3;
    QGridLayout *gridLayout5;
    QVBoxLayout *vboxLayout1;
    QTreeWidget *availableMetricsTreeWidget;
    QLineEdit *availableMetricLineEdit;
    QVBoxLayout *vboxLayout2;
    QSpacerItem *spacerItem15;
    QPushButton *metricInfoButton;
    QSpacerItem *spacerItem16;
    QPushButton *metricSearchButton;
    QSpacerItem *spacerItem17;
    QPushButton *metricDeleteButton;
    QSpacerItem *spacerItem18;
    QPushButton *metricAddButton;
    QSpacerItem *spacerItem19;
    QPushButton *sourceButton;
    QSpacerItem *spacerItem20;
    QWidget *TabPage;
    QGridLayout *gridLayout6;
    QHBoxLayout *hboxLayout5;
    QVBoxLayout *vboxLayout3;
    QPushButton *revertColorButton;
    QedColorShowLabel *revertColorLabel;
    QSpacerItem *spacerItem21;
    QedColorShowLabel *applyColorLabel;
    QPushButton *applyColorButton;
    QHBoxLayout *hboxLayout6;
    QedColorPicker *colorPicker;
    QedColorLuminancePicker *luminancePicker;
    QSpacerItem *spacerItem22;
    QVBoxLayout *vboxLayout4;
    QHBoxLayout *hboxLayout7;
    QGridLayout *gridLayout7;
    QLabel *saturationTextLabel;
    QLabel *redTextLabel;
    QedColSpinBox *rEd;
    QedColSpinBox *bEd;
    QLabel *hueTextLabel;
    QedColSpinBox *gEd;
    QLabel *blueTextLabel;
    QLabel *greenTextLabel;
    QedColSpinBox *sEd;
    QedColSpinBox *hEd;
    QedColSpinBox *vEd;
    QLabel *valueTextLabel;
    QSpacerItem *spacerItem23;
    QGridLayout *gridLayout8;
    QLabel *plotLabelLabel;
    QedColLineEdit *colorLineEdit;
    QLabel *plotColorLabel;
    QSpacerItem *spacerItem24;
    QLineEdit *plotLabelLineEdit;
    QSpacerItem *spacerItem25;
    QVBoxLayout *vboxLayout5;
    QTreeWidget *chartMetricsTreeWidget;
    QLineEdit *chartMetricLineEdit;
    QHBoxLayout *hboxLayout8;
    QSpacerItem *spacerItem26;
    QPushButton *buttonApply;
    QSpacerItem *horizontalSpacer;
    QPushButton *buttonOk;
    QPushButton *buttonCancel;

    void setupUi(QDialog *ChartDialog)
    {
        if (ChartDialog->objectName().isEmpty())
            ChartDialog->setObjectName(QString::fromUtf8("ChartDialog"));
        ChartDialog->resize(612, 457);
        ChartDialog->setFocusPolicy(Qt::WheelFocus);
        const QIcon icon = QIcon(QString::fromUtf8(":/images/pmchart.png"));
        ChartDialog->setWindowIcon(icon);
        ChartDialog->setSizeGripEnabled(true);
        gridLayout = new QGridLayout(ChartDialog);
#ifndef Q_OS_MAC
        gridLayout->setSpacing(6);
#endif
#ifndef Q_OS_MAC
        gridLayout->setContentsMargins(9, 9, 9, 9);
#endif
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        tabWidget = new QTabWidget(ChartDialog);
        tabWidget->setObjectName(QString::fromUtf8("tabWidget"));
        QSizePolicy sizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(tabWidget->sizePolicy().hasHeightForWidth());
        tabWidget->setSizePolicy(sizePolicy);
        Widget2 = new QWidget();
        Widget2->setObjectName(QString::fromUtf8("Widget2"));
        gridLayout1 = new QGridLayout(Widget2);
#ifndef Q_OS_MAC
        gridLayout1->setSpacing(6);
#endif
#ifndef Q_OS_MAC
        gridLayout1->setContentsMargins(9, 9, 9, 9);
#endif
        gridLayout1->setObjectName(QString::fromUtf8("gridLayout1"));
        vboxLayout = new QVBoxLayout();
#ifndef Q_OS_MAC
        vboxLayout->setSpacing(6);
#endif
#ifndef Q_OS_MAC
        vboxLayout->setContentsMargins(0, 0, 0, 0);
#endif
        vboxLayout->setObjectName(QString::fromUtf8("vboxLayout"));
        spacerItem = new QSpacerItem(20, 1, QSizePolicy::Minimum, QSizePolicy::Fixed);

        vboxLayout->addItem(spacerItem);

        gridLayout2 = new QGridLayout();
#ifndef Q_OS_MAC
        gridLayout2->setSpacing(6);
#endif
#ifndef Q_OS_MAC
        gridLayout2->setContentsMargins(0, 0, 0, 0);
#endif
        gridLayout2->setObjectName(QString::fromUtf8("gridLayout2"));
        colorSchemeComboBox = new QComboBox(Widget2);
        colorSchemeComboBox->addItem(QString());
        colorSchemeComboBox->addItem(QString());
        colorSchemeComboBox->setObjectName(QString::fromUtf8("colorSchemeComboBox"));

        gridLayout2->addWidget(colorSchemeComboBox, 2, 1, 1, 1);

        typeComboBox = new QComboBox(Widget2);
        typeComboBox->addItem(QString());
        typeComboBox->addItem(QString());
        typeComboBox->addItem(QString());
        typeComboBox->addItem(QString());
        typeComboBox->addItem(QString());
        typeComboBox->setObjectName(QString::fromUtf8("typeComboBox"));

        gridLayout2->addWidget(typeComboBox, 1, 1, 1, 1);

        colorSchemeLabel = new QLabel(Widget2);
        colorSchemeLabel->setObjectName(QString::fromUtf8("colorSchemeLabel"));
        colorSchemeLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout2->addWidget(colorSchemeLabel, 2, 0, 1, 1);

        hboxLayout = new QHBoxLayout();
#ifndef Q_OS_MAC
        hboxLayout->setSpacing(6);
#endif
        hboxLayout->setContentsMargins(0, 0, 0, 0);
        hboxLayout->setObjectName(QString::fromUtf8("hboxLayout"));
        spacerItem1 = new QSpacerItem(21, 20, QSizePolicy::Fixed, QSizePolicy::Minimum);

        hboxLayout->addItem(spacerItem1);

        legendOn = new QRadioButton(Widget2);
        legendOn->setObjectName(QString::fromUtf8("legendOn"));
        QSizePolicy sizePolicy1(QSizePolicy::Fixed, QSizePolicy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(legendOn->sizePolicy().hasHeightForWidth());
        legendOn->setSizePolicy(sizePolicy1);
        legendOn->setChecked(true);
        legendOn->setAutoExclusive(false);

        hboxLayout->addWidget(legendOn);

        spacerItem2 = new QSpacerItem(16, 20, QSizePolicy::Fixed, QSizePolicy::Minimum);

        hboxLayout->addItem(spacerItem2);

        legendOff = new QRadioButton(Widget2);
        legendOff->setObjectName(QString::fromUtf8("legendOff"));
        sizePolicy1.setHeightForWidth(legendOff->sizePolicy().hasHeightForWidth());
        legendOff->setSizePolicy(sizePolicy1);
        legendOff->setAutoExclusive(false);

        hboxLayout->addWidget(legendOff);

        spacerItem3 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        hboxLayout->addItem(spacerItem3);


        gridLayout2->addLayout(hboxLayout, 3, 1, 1, 1);

        legendTextLabel = new QLabel(Widget2);
        legendTextLabel->setObjectName(QString::fromUtf8("legendTextLabel"));
        legendTextLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        legendTextLabel->setWordWrap(false);

        gridLayout2->addWidget(legendTextLabel, 3, 0, 1, 1);

        antiAliasingTextLabel = new QLabel(Widget2);
        antiAliasingTextLabel->setObjectName(QString::fromUtf8("antiAliasingTextLabel"));
        antiAliasingTextLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        antiAliasingTextLabel->setWordWrap(false);

        gridLayout2->addWidget(antiAliasingTextLabel, 4, 0, 1, 1);

        hboxLayout1 = new QHBoxLayout();
        hboxLayout1->setObjectName(QString::fromUtf8("hboxLayout1"));
        spacerItem4 = new QSpacerItem(21, 20, QSizePolicy::Fixed, QSizePolicy::Minimum);

        hboxLayout1->addItem(spacerItem4);

        antiAliasingAuto = new QRadioButton(Widget2);
        antiAliasingAuto->setObjectName(QString::fromUtf8("antiAliasingAuto"));
        QSizePolicy sizePolicy2(QSizePolicy::Fixed, QSizePolicy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(antiAliasingAuto->sizePolicy().hasHeightForWidth());
        antiAliasingAuto->setSizePolicy(sizePolicy2);
        antiAliasingAuto->setChecked(true);
        antiAliasingAuto->setAutoExclusive(false);

        hboxLayout1->addWidget(antiAliasingAuto);

        spacerItem5 = new QSpacerItem(14, 20, QSizePolicy::Fixed, QSizePolicy::Minimum);

        hboxLayout1->addItem(spacerItem5);

        antiAliasingOn = new QRadioButton(Widget2);
        antiAliasingOn->setObjectName(QString::fromUtf8("antiAliasingOn"));
        sizePolicy1.setHeightForWidth(antiAliasingOn->sizePolicy().hasHeightForWidth());
        antiAliasingOn->setSizePolicy(sizePolicy1);
        antiAliasingOn->setAutoExclusive(false);

        hboxLayout1->addWidget(antiAliasingOn);

        spacerItem6 = new QSpacerItem(21, 20, QSizePolicy::Fixed, QSizePolicy::Minimum);

        hboxLayout1->addItem(spacerItem6);

        antiAliasingOff = new QRadioButton(Widget2);
        antiAliasingOff->setObjectName(QString::fromUtf8("antiAliasingOff"));
        sizePolicy1.setHeightForWidth(antiAliasingOff->sizePolicy().hasHeightForWidth());
        antiAliasingOff->setSizePolicy(sizePolicy1);
        antiAliasingOff->setAutoExclusive(false);

        hboxLayout1->addWidget(antiAliasingOff);

        spacerItem7 = new QSpacerItem(5, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        hboxLayout1->addItem(spacerItem7);


        gridLayout2->addLayout(hboxLayout1, 4, 1, 1, 1);

        titleLineEdit = new QLineEdit(Widget2);
        titleLineEdit->setObjectName(QString::fromUtf8("titleLineEdit"));

        gridLayout2->addWidget(titleLineEdit, 0, 1, 1, 1);

        titleTextLabel = new QLabel(Widget2);
        titleTextLabel->setObjectName(QString::fromUtf8("titleTextLabel"));
        titleTextLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        titleTextLabel->setWordWrap(false);

        gridLayout2->addWidget(titleTextLabel, 0, 0, 1, 1);

        styleLabel = new QLabel(Widget2);
        styleLabel->setObjectName(QString::fromUtf8("styleLabel"));
        styleLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        styleLabel->setWordWrap(false);

        gridLayout2->addWidget(styleLabel, 1, 0, 1, 1);


        vboxLayout->addLayout(gridLayout2);

        spacerItem8 = new QSpacerItem(20, 16, QSizePolicy::Minimum, QSizePolicy::Fixed);

        vboxLayout->addItem(spacerItem8);

        yAxisGroupBox = new QGroupBox(Widget2);
        yAxisGroupBox->setObjectName(QString::fromUtf8("yAxisGroupBox"));
        QSizePolicy sizePolicy3(QSizePolicy::Expanding, QSizePolicy::Expanding);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(yAxisGroupBox->sizePolicy().hasHeightForWidth());
        yAxisGroupBox->setSizePolicy(sizePolicy3);
        gridLayout3 = new QGridLayout(yAxisGroupBox);
#ifndef Q_OS_MAC
        gridLayout3->setSpacing(6);
#endif
#ifndef Q_OS_MAC
        gridLayout3->setContentsMargins(9, 9, 9, 9);
#endif
        gridLayout3->setObjectName(QString::fromUtf8("gridLayout3"));
        gridLayout4 = new QGridLayout();
#ifndef Q_OS_MAC
        gridLayout4->setSpacing(6);
#endif
#ifndef Q_OS_MAC
        gridLayout4->setContentsMargins(0, 0, 0, 0);
#endif
        gridLayout4->setObjectName(QString::fromUtf8("gridLayout4"));
        maxTextLabel = new QLabel(yAxisGroupBox);
        maxTextLabel->setObjectName(QString::fromUtf8("maxTextLabel"));
        maxTextLabel->setEnabled(false);
        maxTextLabel->setWordWrap(false);

        gridLayout4->addWidget(maxTextLabel, 2, 0, 1, 1);

        hboxLayout2 = new QHBoxLayout();
#ifndef Q_OS_MAC
        hboxLayout2->setSpacing(6);
#endif
        hboxLayout2->setContentsMargins(0, 0, 0, 0);
        hboxLayout2->setObjectName(QString::fromUtf8("hboxLayout2"));
        yAxisMinimum = new QDoubleSpinBox(yAxisGroupBox);
        yAxisMinimum->setObjectName(QString::fromUtf8("yAxisMinimum"));
        yAxisMinimum->setEnabled(false);
        yAxisMinimum->setMaximum(1000000000.000000000000000);

        hboxLayout2->addWidget(yAxisMinimum);

        spacerItem9 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        hboxLayout2->addItem(spacerItem9);


        gridLayout4->addLayout(hboxLayout2, 1, 1, 1, 1);

        minTextLabel = new QLabel(yAxisGroupBox);
        minTextLabel->setObjectName(QString::fromUtf8("minTextLabel"));
        minTextLabel->setEnabled(false);
        minTextLabel->setWordWrap(false);

        gridLayout4->addWidget(minTextLabel, 1, 0, 1, 1);

        hboxLayout3 = new QHBoxLayout();
#ifndef Q_OS_MAC
        hboxLayout3->setSpacing(6);
#endif
        hboxLayout3->setContentsMargins(0, 0, 0, 0);
        hboxLayout3->setObjectName(QString::fromUtf8("hboxLayout3"));
        spacerItem10 = new QSpacerItem(21, 20, QSizePolicy::Fixed, QSizePolicy::Minimum);

        hboxLayout3->addItem(spacerItem10);

        autoScaleOn = new QRadioButton(yAxisGroupBox);
        autoScaleOn->setObjectName(QString::fromUtf8("autoScaleOn"));
        sizePolicy1.setHeightForWidth(autoScaleOn->sizePolicy().hasHeightForWidth());
        autoScaleOn->setSizePolicy(sizePolicy1);
        autoScaleOn->setChecked(true);

        hboxLayout3->addWidget(autoScaleOn);

        spacerItem11 = new QSpacerItem(16, 20, QSizePolicy::Fixed, QSizePolicy::Minimum);

        hboxLayout3->addItem(spacerItem11);

        autoScaleOff = new QRadioButton(yAxisGroupBox);
        autoScaleOff->setObjectName(QString::fromUtf8("autoScaleOff"));
        sizePolicy1.setHeightForWidth(autoScaleOff->sizePolicy().hasHeightForWidth());
        autoScaleOff->setSizePolicy(sizePolicy1);

        hboxLayout3->addWidget(autoScaleOff);

        spacerItem12 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        hboxLayout3->addItem(spacerItem12);


        gridLayout4->addLayout(hboxLayout3, 0, 1, 1, 1);

        hboxLayout4 = new QHBoxLayout();
#ifndef Q_OS_MAC
        hboxLayout4->setSpacing(6);
#endif
        hboxLayout4->setContentsMargins(0, 0, 0, 0);
        hboxLayout4->setObjectName(QString::fromUtf8("hboxLayout4"));
        yAxisMaximum = new QDoubleSpinBox(yAxisGroupBox);
        yAxisMaximum->setObjectName(QString::fromUtf8("yAxisMaximum"));
        yAxisMaximum->setEnabled(false);
        yAxisMaximum->setMaximum(1000000000.000000000000000);

        hboxLayout4->addWidget(yAxisMaximum);

        spacerItem13 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        hboxLayout4->addItem(spacerItem13);


        gridLayout4->addLayout(hboxLayout4, 2, 1, 1, 1);

        autoScale = new QLabel(yAxisGroupBox);
        autoScale->setObjectName(QString::fromUtf8("autoScale"));
        sizePolicy1.setHeightForWidth(autoScale->sizePolicy().hasHeightForWidth());
        autoScale->setSizePolicy(sizePolicy1);
        autoScale->setWordWrap(false);

        gridLayout4->addWidget(autoScale, 0, 0, 1, 1);


        gridLayout3->addLayout(gridLayout4, 0, 0, 1, 1);

        rateConvertCheckBox = new QCheckBox(yAxisGroupBox);
        rateConvertCheckBox->setObjectName(QString::fromUtf8("rateConvertCheckBox"));
        rateConvertCheckBox->setChecked(true);

        gridLayout3->addWidget(rateConvertCheckBox, 1, 0, 1, 1);


        vboxLayout->addWidget(yAxisGroupBox);

        spacerItem14 = new QSpacerItem(20, 16, QSizePolicy::Minimum, QSizePolicy::MinimumExpanding);

        vboxLayout->addItem(spacerItem14);


        gridLayout1->addLayout(vboxLayout, 0, 0, 1, 1);

        tabWidget->addTab(Widget2, QString());
        Widget3 = new QWidget();
        Widget3->setObjectName(QString::fromUtf8("Widget3"));
        gridLayout5 = new QGridLayout(Widget3);
#ifndef Q_OS_MAC
        gridLayout5->setSpacing(6);
#endif
#ifndef Q_OS_MAC
        gridLayout5->setContentsMargins(9, 9, 9, 9);
#endif
        gridLayout5->setObjectName(QString::fromUtf8("gridLayout5"));
        vboxLayout1 = new QVBoxLayout();
        vboxLayout1->setSpacing(0);
#ifndef Q_OS_MAC
        vboxLayout1->setContentsMargins(0, 0, 0, 0);
#endif
        vboxLayout1->setObjectName(QString::fromUtf8("vboxLayout1"));
        availableMetricsTreeWidget = new QTreeWidget(Widget3);
        availableMetricsTreeWidget->setObjectName(QString::fromUtf8("availableMetricsTreeWidget"));
        QFont font;
        font.setPointSize(10);
        availableMetricsTreeWidget->setFont(font);
        availableMetricsTreeWidget->setCursor(QCursor(static_cast<Qt::CursorShape>(0)));
        availableMetricsTreeWidget->setMouseTracking(true);
        availableMetricsTreeWidget->setFrameShape(QFrame::StyledPanel);
        availableMetricsTreeWidget->setVerticalScrollBarPolicy(Qt::ScrollBarAsNeeded);
        availableMetricsTreeWidget->setAlternatingRowColors(true);
        availableMetricsTreeWidget->setSelectionMode(QAbstractItemView::ExtendedSelection);
        availableMetricsTreeWidget->setRootIsDecorated(false);
        availableMetricsTreeWidget->setUniformRowHeights(true);
        availableMetricsTreeWidget->setColumnCount(1);

        vboxLayout1->addWidget(availableMetricsTreeWidget);

        availableMetricLineEdit = new QLineEdit(Widget3);
        availableMetricLineEdit->setObjectName(QString::fromUtf8("availableMetricLineEdit"));
        QFont font1;
        font1.setPointSize(10);
        font1.setBold(true);
        font1.setWeight(75);
        availableMetricLineEdit->setFont(font1);
        availableMetricLineEdit->setReadOnly(true);

        vboxLayout1->addWidget(availableMetricLineEdit);


        gridLayout5->addLayout(vboxLayout1, 0, 1, 1, 1);

        vboxLayout2 = new QVBoxLayout();
        vboxLayout2->setSpacing(0);
        vboxLayout2->setContentsMargins(0, 0, 0, 0);
        vboxLayout2->setObjectName(QString::fromUtf8("vboxLayout2"));
        spacerItem15 = new QSpacerItem(20, 16, QSizePolicy::Minimum, QSizePolicy::Fixed);

        vboxLayout2->addItem(spacerItem15);

        metricInfoButton = new QPushButton(Widget3);
        metricInfoButton->setObjectName(QString::fromUtf8("metricInfoButton"));
        sizePolicy1.setHeightForWidth(metricInfoButton->sizePolicy().hasHeightForWidth());
        metricInfoButton->setSizePolicy(sizePolicy1);
        metricInfoButton->setMaximumSize(QSize(32, 32));
        const QIcon icon1 = QIcon(QString::fromUtf8(":/images/help-browser.png"));
        metricInfoButton->setIcon(icon1);

        vboxLayout2->addWidget(metricInfoButton);

        spacerItem16 = new QSpacerItem(20, 10, QSizePolicy::Minimum, QSizePolicy::Fixed);

        vboxLayout2->addItem(spacerItem16);

        metricSearchButton = new QPushButton(Widget3);
        metricSearchButton->setObjectName(QString::fromUtf8("metricSearchButton"));
        metricSearchButton->setEnabled(true);
        sizePolicy1.setHeightForWidth(metricSearchButton->sizePolicy().hasHeightForWidth());
        metricSearchButton->setSizePolicy(sizePolicy1);
        metricSearchButton->setMaximumSize(QSize(32, 32));
        const QIcon icon2 = QIcon(QString::fromUtf8(":/images/system-search.png"));
        metricSearchButton->setIcon(icon2);

        vboxLayout2->addWidget(metricSearchButton);

        spacerItem17 = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        vboxLayout2->addItem(spacerItem17);

        metricDeleteButton = new QPushButton(Widget3);
        metricDeleteButton->setObjectName(QString::fromUtf8("metricDeleteButton"));
        sizePolicy1.setHeightForWidth(metricDeleteButton->sizePolicy().hasHeightForWidth());
        metricDeleteButton->setSizePolicy(sizePolicy1);
        metricDeleteButton->setMinimumSize(QSize(32, 32));
        metricDeleteButton->setMaximumSize(QSize(32, 32));
        const QIcon icon3 = QIcon(QString::fromUtf8(":/images/process-stop.png"));
        metricDeleteButton->setIcon(icon3);

        vboxLayout2->addWidget(metricDeleteButton);

        spacerItem18 = new QSpacerItem(20, 10, QSizePolicy::Minimum, QSizePolicy::Fixed);

        vboxLayout2->addItem(spacerItem18);

        metricAddButton = new QPushButton(Widget3);
        metricAddButton->setObjectName(QString::fromUtf8("metricAddButton"));
        sizePolicy1.setHeightForWidth(metricAddButton->sizePolicy().hasHeightForWidth());
        metricAddButton->setSizePolicy(sizePolicy1);
        metricAddButton->setMinimumSize(QSize(32, 32));
        metricAddButton->setMaximumSize(QSize(32, 32));
        const QIcon icon4 = QIcon(QString::fromUtf8(":/images/go-previous.png"));
        metricAddButton->setIcon(icon4);

        vboxLayout2->addWidget(metricAddButton);

        spacerItem19 = new QSpacerItem(20, 41, QSizePolicy::Minimum, QSizePolicy::Expanding);

        vboxLayout2->addItem(spacerItem19);

        sourceButton = new QPushButton(Widget3);
        sourceButton->setObjectName(QString::fromUtf8("sourceButton"));
        sizePolicy1.setHeightForWidth(sourceButton->sizePolicy().hasHeightForWidth());
        sourceButton->setSizePolicy(sizePolicy1);
        sourceButton->setMinimumSize(QSize(32, 32));
        sourceButton->setMaximumSize(QSize(32, 32));

        vboxLayout2->addWidget(sourceButton);

        spacerItem20 = new QSpacerItem(20, 51, QSizePolicy::Minimum, QSizePolicy::Fixed);

        vboxLayout2->addItem(spacerItem20);


        gridLayout5->addLayout(vboxLayout2, 0, 0, 1, 1);

        tabWidget->addTab(Widget3, QString());
        TabPage = new QWidget();
        TabPage->setObjectName(QString::fromUtf8("TabPage"));
        gridLayout6 = new QGridLayout(TabPage);
#ifndef Q_OS_MAC
        gridLayout6->setSpacing(6);
#endif
#ifndef Q_OS_MAC
        gridLayout6->setContentsMargins(9, 9, 9, 9);
#endif
        gridLayout6->setObjectName(QString::fromUtf8("gridLayout6"));
        hboxLayout5 = new QHBoxLayout();
#ifndef Q_OS_MAC
        hboxLayout5->setSpacing(6);
#endif
#ifndef Q_OS_MAC
        hboxLayout5->setContentsMargins(0, 0, 0, 0);
#endif
        hboxLayout5->setObjectName(QString::fromUtf8("hboxLayout5"));
        vboxLayout3 = new QVBoxLayout();
        vboxLayout3->setSpacing(0);
#ifndef Q_OS_MAC
        vboxLayout3->setContentsMargins(0, 0, 0, 0);
#endif
        vboxLayout3->setObjectName(QString::fromUtf8("vboxLayout3"));
        revertColorButton = new QPushButton(TabPage);
        revertColorButton->setObjectName(QString::fromUtf8("revertColorButton"));
        sizePolicy1.setHeightForWidth(revertColorButton->sizePolicy().hasHeightForWidth());
        revertColorButton->setSizePolicy(sizePolicy1);
        revertColorButton->setMinimumSize(QSize(32, 32));
        revertColorButton->setMaximumSize(QSize(32, 32));
        const QIcon icon5 = QIcon(QString::fromUtf8(":/images/go-jump.png"));
        revertColorButton->setIcon(icon5);

        vboxLayout3->addWidget(revertColorButton);

        revertColorLabel = new QedColorShowLabel(TabPage);
        revertColorLabel->setObjectName(QString::fromUtf8("revertColorLabel"));
        sizePolicy1.setHeightForWidth(revertColorLabel->sizePolicy().hasHeightForWidth());
        revertColorLabel->setSizePolicy(sizePolicy1);
        revertColorLabel->setMinimumSize(QSize(32, 32));
        revertColorLabel->setMaximumSize(QSize(32, 32));
        revertColorLabel->setFrameShape(QFrame::Panel);
        revertColorLabel->setFrameShadow(QFrame::Sunken);
        revertColorLabel->setLineWidth(2);

        vboxLayout3->addWidget(revertColorLabel);

        spacerItem21 = new QSpacerItem(20, 8, QSizePolicy::Minimum, QSizePolicy::Fixed);

        vboxLayout3->addItem(spacerItem21);

        applyColorLabel = new QedColorShowLabel(TabPage);
        applyColorLabel->setObjectName(QString::fromUtf8("applyColorLabel"));
        sizePolicy1.setHeightForWidth(applyColorLabel->sizePolicy().hasHeightForWidth());
        applyColorLabel->setSizePolicy(sizePolicy1);
        applyColorLabel->setMinimumSize(QSize(32, 32));
        applyColorLabel->setMaximumSize(QSize(32, 32));
        applyColorLabel->setFrameShape(QFrame::Panel);
        applyColorLabel->setFrameShadow(QFrame::Sunken);
        applyColorLabel->setLineWidth(2);

        vboxLayout3->addWidget(applyColorLabel);

        applyColorButton = new QPushButton(TabPage);
        applyColorButton->setObjectName(QString::fromUtf8("applyColorButton"));
        sizePolicy1.setHeightForWidth(applyColorButton->sizePolicy().hasHeightForWidth());
        applyColorButton->setSizePolicy(sizePolicy1);
        applyColorButton->setMinimumSize(QSize(32, 32));
        applyColorButton->setMaximumSize(QSize(32, 32));
        applyColorButton->setIcon(icon4);

        vboxLayout3->addWidget(applyColorButton);


        hboxLayout5->addLayout(vboxLayout3);

        hboxLayout6 = new QHBoxLayout();
#ifndef Q_OS_MAC
        hboxLayout6->setSpacing(6);
#endif
        hboxLayout6->setContentsMargins(0, 0, 0, 0);
        hboxLayout6->setObjectName(QString::fromUtf8("hboxLayout6"));
        colorPicker = new QedColorPicker(TabPage);
        colorPicker->setObjectName(QString::fromUtf8("colorPicker"));
        sizePolicy1.setHeightForWidth(colorPicker->sizePolicy().hasHeightForWidth());
        colorPicker->setSizePolicy(sizePolicy1);
        colorPicker->setMinimumSize(QSize(176, 176));
        colorPicker->setMaximumSize(QSize(176, 176));
        colorPicker->setFrameShape(QFrame::Panel);
        colorPicker->setFrameShadow(QFrame::Sunken);
        colorPicker->setLineWidth(2);

        hboxLayout6->addWidget(colorPicker);

        luminancePicker = new QedColorLuminancePicker(TabPage);
        luminancePicker->setObjectName(QString::fromUtf8("luminancePicker"));
        sizePolicy1.setHeightForWidth(luminancePicker->sizePolicy().hasHeightForWidth());
        luminancePicker->setSizePolicy(sizePolicy1);
        luminancePicker->setMinimumSize(QSize(24, 176));
        luminancePicker->setMaximumSize(QSize(24, 176));

        hboxLayout6->addWidget(luminancePicker);


        hboxLayout5->addLayout(hboxLayout6);

        spacerItem22 = new QSpacerItem(16, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        hboxLayout5->addItem(spacerItem22);


        gridLayout6->addLayout(hboxLayout5, 0, 0, 1, 1);

        vboxLayout4 = new QVBoxLayout();
#ifndef Q_OS_MAC
        vboxLayout4->setSpacing(6);
#endif
        vboxLayout4->setContentsMargins(0, 0, 0, 0);
        vboxLayout4->setObjectName(QString::fromUtf8("vboxLayout4"));
        hboxLayout7 = new QHBoxLayout();
#ifndef Q_OS_MAC
        hboxLayout7->setSpacing(6);
#endif
#ifndef Q_OS_MAC
        hboxLayout7->setContentsMargins(0, 0, 0, 0);
#endif
        hboxLayout7->setObjectName(QString::fromUtf8("hboxLayout7"));
        gridLayout7 = new QGridLayout();
#ifndef Q_OS_MAC
        gridLayout7->setSpacing(6);
#endif
#ifndef Q_OS_MAC
        gridLayout7->setContentsMargins(0, 0, 0, 0);
#endif
        gridLayout7->setObjectName(QString::fromUtf8("gridLayout7"));
        saturationTextLabel = new QLabel(TabPage);
        saturationTextLabel->setObjectName(QString::fromUtf8("saturationTextLabel"));
        saturationTextLabel->setFont(font);
        saturationTextLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        saturationTextLabel->setWordWrap(false);

        gridLayout7->addWidget(saturationTextLabel, 0, 2, 1, 1);

        redTextLabel = new QLabel(TabPage);
        redTextLabel->setObjectName(QString::fromUtf8("redTextLabel"));
        redTextLabel->setFont(font);
        redTextLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        redTextLabel->setWordWrap(false);

        gridLayout7->addWidget(redTextLabel, 1, 0, 1, 1);

        rEd = new QedColSpinBox(TabPage);
        rEd->setObjectName(QString::fromUtf8("rEd"));
        rEd->setAlignment(Qt::AlignRight);

        gridLayout7->addWidget(rEd, 1, 1, 1, 1);

        bEd = new QedColSpinBox(TabPage);
        bEd->setObjectName(QString::fromUtf8("bEd"));
        bEd->setAlignment(Qt::AlignRight);

        gridLayout7->addWidget(bEd, 1, 5, 1, 1);

        hueTextLabel = new QLabel(TabPage);
        hueTextLabel->setObjectName(QString::fromUtf8("hueTextLabel"));
        hueTextLabel->setFont(font);
        hueTextLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        hueTextLabel->setWordWrap(false);

        gridLayout7->addWidget(hueTextLabel, 0, 0, 1, 1);

        gEd = new QedColSpinBox(TabPage);
        gEd->setObjectName(QString::fromUtf8("gEd"));
        gEd->setAlignment(Qt::AlignRight);

        gridLayout7->addWidget(gEd, 1, 3, 1, 1);

        blueTextLabel = new QLabel(TabPage);
        blueTextLabel->setObjectName(QString::fromUtf8("blueTextLabel"));
        blueTextLabel->setFont(font);
        blueTextLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        blueTextLabel->setWordWrap(false);

        gridLayout7->addWidget(blueTextLabel, 1, 4, 1, 1);

        greenTextLabel = new QLabel(TabPage);
        greenTextLabel->setObjectName(QString::fromUtf8("greenTextLabel"));
        greenTextLabel->setFont(font);
        greenTextLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        greenTextLabel->setWordWrap(false);

        gridLayout7->addWidget(greenTextLabel, 1, 2, 1, 1);

        sEd = new QedColSpinBox(TabPage);
        sEd->setObjectName(QString::fromUtf8("sEd"));
        sEd->setAlignment(Qt::AlignRight);

        gridLayout7->addWidget(sEd, 0, 3, 1, 1);

        hEd = new QedColSpinBox(TabPage);
        hEd->setObjectName(QString::fromUtf8("hEd"));
        hEd->setAlignment(Qt::AlignRight);

        gridLayout7->addWidget(hEd, 0, 1, 1, 1);

        vEd = new QedColSpinBox(TabPage);
        vEd->setObjectName(QString::fromUtf8("vEd"));
        vEd->setAlignment(Qt::AlignRight);

        gridLayout7->addWidget(vEd, 0, 5, 1, 1);

        valueTextLabel = new QLabel(TabPage);
        valueTextLabel->setObjectName(QString::fromUtf8("valueTextLabel"));
        valueTextLabel->setFont(font);
        valueTextLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        valueTextLabel->setWordWrap(false);

        gridLayout7->addWidget(valueTextLabel, 0, 4, 1, 1);


        hboxLayout7->addLayout(gridLayout7);

        spacerItem23 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        hboxLayout7->addItem(spacerItem23);


        vboxLayout4->addLayout(hboxLayout7);

        gridLayout8 = new QGridLayout();
#ifndef Q_OS_MAC
        gridLayout8->setSpacing(6);
#endif
        gridLayout8->setContentsMargins(0, 0, 0, 0);
        gridLayout8->setObjectName(QString::fromUtf8("gridLayout8"));
        plotLabelLabel = new QLabel(TabPage);
        plotLabelLabel->setObjectName(QString::fromUtf8("plotLabelLabel"));

        gridLayout8->addWidget(plotLabelLabel, 1, 0, 1, 1);

        colorLineEdit = new QedColLineEdit(TabPage);
        colorLineEdit->setObjectName(QString::fromUtf8("colorLineEdit"));
        sizePolicy1.setHeightForWidth(colorLineEdit->sizePolicy().hasHeightForWidth());
        colorLineEdit->setSizePolicy(sizePolicy1);

        gridLayout8->addWidget(colorLineEdit, 0, 1, 1, 1);

        plotColorLabel = new QLabel(TabPage);
        plotColorLabel->setObjectName(QString::fromUtf8("plotColorLabel"));

        gridLayout8->addWidget(plotColorLabel, 0, 0, 1, 1);

        spacerItem24 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        gridLayout8->addItem(spacerItem24, 0, 2, 1, 1);

        plotLabelLineEdit = new QLineEdit(TabPage);
        plotLabelLineEdit->setObjectName(QString::fromUtf8("plotLabelLineEdit"));

        gridLayout8->addWidget(plotLabelLineEdit, 1, 1, 1, 2);


        vboxLayout4->addLayout(gridLayout8);

        spacerItem25 = new QSpacerItem(20, 20, QSizePolicy::Minimum, QSizePolicy::Expanding);

        vboxLayout4->addItem(spacerItem25);


        gridLayout6->addLayout(vboxLayout4, 1, 0, 1, 1);

        tabWidget->addTab(TabPage, QString());

        gridLayout->addWidget(tabWidget, 0, 1, 1, 1);

        vboxLayout5 = new QVBoxLayout();
        vboxLayout5->setSpacing(0);
        vboxLayout5->setContentsMargins(0, 0, 0, 0);
        vboxLayout5->setObjectName(QString::fromUtf8("vboxLayout5"));
        chartMetricsTreeWidget = new QTreeWidget(ChartDialog);
        chartMetricsTreeWidget->setObjectName(QString::fromUtf8("chartMetricsTreeWidget"));
        sizePolicy.setHeightForWidth(chartMetricsTreeWidget->sizePolicy().hasHeightForWidth());
        chartMetricsTreeWidget->setSizePolicy(sizePolicy);
        chartMetricsTreeWidget->setFont(font);
        chartMetricsTreeWidget->setCursor(QCursor(static_cast<Qt::CursorShape>(0)));
        chartMetricsTreeWidget->setMouseTracking(true);
        chartMetricsTreeWidget->setFrameShape(QFrame::StyledPanel);
        chartMetricsTreeWidget->setVerticalScrollBarPolicy(Qt::ScrollBarAsNeeded);
        chartMetricsTreeWidget->setAlternatingRowColors(true);
        chartMetricsTreeWidget->setSelectionMode(QAbstractItemView::ExtendedSelection);
        chartMetricsTreeWidget->setRootIsDecorated(false);

        vboxLayout5->addWidget(chartMetricsTreeWidget);

        chartMetricLineEdit = new QLineEdit(ChartDialog);
        chartMetricLineEdit->setObjectName(QString::fromUtf8("chartMetricLineEdit"));
        QSizePolicy sizePolicy4(QSizePolicy::Expanding, QSizePolicy::Fixed);
        sizePolicy4.setHorizontalStretch(0);
        sizePolicy4.setVerticalStretch(0);
        sizePolicy4.setHeightForWidth(chartMetricLineEdit->sizePolicy().hasHeightForWidth());
        chartMetricLineEdit->setSizePolicy(sizePolicy4);
        chartMetricLineEdit->setFont(font1);
        chartMetricLineEdit->setReadOnly(true);

        vboxLayout5->addWidget(chartMetricLineEdit);


        gridLayout->addLayout(vboxLayout5, 0, 0, 1, 1);

        hboxLayout8 = new QHBoxLayout();
#ifndef Q_OS_MAC
        hboxLayout8->setSpacing(6);
#endif
        hboxLayout8->setContentsMargins(0, 0, 0, 0);
        hboxLayout8->setObjectName(QString::fromUtf8("hboxLayout8"));
        spacerItem26 = new QSpacerItem(20, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        hboxLayout8->addItem(spacerItem26);

        buttonApply = new QPushButton(ChartDialog);
        buttonApply->setObjectName(QString::fromUtf8("buttonApply"));

        hboxLayout8->addWidget(buttonApply);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Fixed, QSizePolicy::Minimum);

        hboxLayout8->addItem(horizontalSpacer);

        buttonOk = new QPushButton(ChartDialog);
        buttonOk->setObjectName(QString::fromUtf8("buttonOk"));
        buttonOk->setAutoDefault(true);

        hboxLayout8->addWidget(buttonOk);

        buttonCancel = new QPushButton(ChartDialog);
        buttonCancel->setObjectName(QString::fromUtf8("buttonCancel"));
        buttonCancel->setAutoDefault(true);

        hboxLayout8->addWidget(buttonCancel);


        gridLayout->addLayout(hboxLayout8, 1, 0, 1, 2);


        retranslateUi(ChartDialog);

        tabWidget->setCurrentIndex(0);
        buttonOk->setDefault(true);


        QMetaObject::connectSlotsByName(ChartDialog);
    } // setupUi

    void retranslateUi(QDialog *ChartDialog)
    {
        ChartDialog->setWindowTitle(QCoreApplication::translate("ChartDialog", "New Chart", nullptr));
        colorSchemeComboBox->setItemText(0, QCoreApplication::translate("ChartDialog", "Default Scheme", nullptr));
        colorSchemeComboBox->setItemText(1, QCoreApplication::translate("ChartDialog", "New Scheme...", nullptr));

#if QT_CONFIG(whatsthis)
        colorSchemeComboBox->setWhatsThis(QCoreApplication::translate("ChartDialog", "Drop down llist of available color schemes.  The color scheme determines the initial color for each new plot added to a chart.", nullptr));
#endif // QT_CONFIG(whatsthis)
        typeComboBox->setItemText(0, QCoreApplication::translate("ChartDialog", "Line Plot", nullptr));
        typeComboBox->setItemText(1, QCoreApplication::translate("ChartDialog", "Bar Plot", nullptr));
        typeComboBox->setItemText(2, QCoreApplication::translate("ChartDialog", "Stacked Bar", nullptr));
        typeComboBox->setItemText(3, QCoreApplication::translate("ChartDialog", "Area Plot", nullptr));
        typeComboBox->setItemText(4, QCoreApplication::translate("ChartDialog", "Utilization", nullptr));

#if QT_CONFIG(whatsthis)
        typeComboBox->setWhatsThis(QCoreApplication::translate("ChartDialog", "Drop down list of chart styles, specifying how the values in a chart should be displayed", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(whatsthis)
        colorSchemeLabel->setWhatsThis(QCoreApplication::translate("ChartDialog", "Drop down llist of available color schemes.  The color scheme determines the initial color for each new plot added to a chart.", nullptr));
#endif // QT_CONFIG(whatsthis)
        colorSchemeLabel->setText(QCoreApplication::translate("ChartDialog", "Colors:", nullptr));
#if QT_CONFIG(whatsthis)
        legendOn->setWhatsThis(QCoreApplication::translate("ChartDialog", "Enables or disables the chart legend, which shows the mapping between plot names and colors", nullptr));
#endif // QT_CONFIG(whatsthis)
        legendOn->setText(QCoreApplication::translate("ChartDialog", "On", nullptr));
#if QT_CONFIG(whatsthis)
        legendOff->setWhatsThis(QCoreApplication::translate("ChartDialog", "Enables or disables the chart legend, which  shows the mapping between plot names and colors", nullptr));
#endif // QT_CONFIG(whatsthis)
        legendOff->setText(QCoreApplication::translate("ChartDialog", "Off", nullptr));
#if QT_CONFIG(whatsthis)
        legendTextLabel->setWhatsThis(QCoreApplication::translate("ChartDialog", "Enables or disables the chart legend, which  shows the mapping between plot names and colors", nullptr));
#endif // QT_CONFIG(whatsthis)
        legendTextLabel->setText(QCoreApplication::translate("ChartDialog", "Legend:", nullptr));
#if QT_CONFIG(whatsthis)
        antiAliasingTextLabel->setWhatsThis(QCoreApplication::translate("ChartDialog", "Enable or disable chart rendering with antialiasing.  This affects the way in which charts are drawn, in particular line plots are less \"blocky\" when rendered this way.  Also, the black like around bar plots are drawn more accurately with antialiasing enabled.", nullptr));
#endif // QT_CONFIG(whatsthis)
        antiAliasingTextLabel->setText(QCoreApplication::translate("ChartDialog", "Antialiasing:", nullptr));
#if QT_CONFIG(whatsthis)
        antiAliasingAuto->setWhatsThis(QCoreApplication::translate("ChartDialog", "Enable or disable chart rendering with antialiasing automatically, based on the chart style chosen.  Line plots will be drawn anti-aliased as this gives a color exactly matching that requested, and bar plots are drawn anti-aliased so their outlines are correctly rendered.", nullptr));
#endif // QT_CONFIG(whatsthis)
        antiAliasingAuto->setText(QCoreApplication::translate("ChartDialog", "Auto", nullptr));
#if QT_CONFIG(whatsthis)
        antiAliasingOn->setWhatsThis(QCoreApplication::translate("ChartDialog", "Enable or disable chart rendering with antialiasing.  This affects the way in which charts are drawn, in particular line plots are less \"blocky\" when rendered this way.  Also, the black like around bar plots are drawn more accurately with antialiasing enabled.", nullptr));
#endif // QT_CONFIG(whatsthis)
        antiAliasingOn->setText(QCoreApplication::translate("ChartDialog", "On", nullptr));
#if QT_CONFIG(whatsthis)
        antiAliasingOff->setWhatsThis(QCoreApplication::translate("ChartDialog", "Enable or disable chart rendering with antialiasing.  This affects the way in which charts are drawn, in particular line plots are less \"blocky\" when rendered this way.  Also, the black like around bar plots are drawn more accurately with antialiasing enabled.", nullptr));
#endif // QT_CONFIG(whatsthis)
        antiAliasingOff->setText(QCoreApplication::translate("ChartDialog", "Off", nullptr));
#if QT_CONFIG(whatsthis)
        titleLineEdit->setWhatsThis(QCoreApplication::translate("ChartDialog", "Add or modifiy the chart title (description)", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(whatsthis)
        titleTextLabel->setWhatsThis(QCoreApplication::translate("ChartDialog", "Add or modifiy the chart title (description)", nullptr));
#endif // QT_CONFIG(whatsthis)
        titleTextLabel->setText(QCoreApplication::translate("ChartDialog", "Title:", nullptr));
#if QT_CONFIG(whatsthis)
        styleLabel->setWhatsThis(QCoreApplication::translate("ChartDialog", "Drop down list of chart styles, specifying how the values in a chart should be displayed", nullptr));
#endif // QT_CONFIG(whatsthis)
        styleLabel->setText(QCoreApplication::translate("ChartDialog", "Style:", nullptr));
#if QT_CONFIG(whatsthis)
        yAxisGroupBox->setWhatsThis(QCoreApplication::translate("ChartDialog", "When enabled, autoscaling causes the Values axis to be scaled based on the minimum and maximum observed values.  When disabled, the minimum and maximum points on the Values Axis specified here are always used (even if the values observed lie outside that range).", nullptr));
#endif // QT_CONFIG(whatsthis)
        yAxisGroupBox->setTitle(QCoreApplication::translate("ChartDialog", "Values Axis", nullptr));
        maxTextLabel->setText(QCoreApplication::translate("ChartDialog", "Maximum:", nullptr));
        minTextLabel->setText(QCoreApplication::translate("ChartDialog", "Minimum:", nullptr));
        autoScaleOn->setText(QCoreApplication::translate("ChartDialog", "On", nullptr));
        autoScaleOff->setText(QCoreApplication::translate("ChartDialog", "Off", nullptr));
        autoScale->setText(QCoreApplication::translate("ChartDialog", "Auto-Scale:", nullptr));
        rateConvertCheckBox->setText(QCoreApplication::translate("ChartDialog", "Rate convert counter metrics", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(Widget2), QCoreApplication::translate("ChartDialog", "Chart", nullptr));
        QTreeWidgetItem *___qtreewidgetitem = availableMetricsTreeWidget->headerItem();
        ___qtreewidgetitem->setText(0, QCoreApplication::translate("ChartDialog", "Available Metrics", nullptr));
#if QT_CONFIG(whatsthis)
        availableMetricsTreeWidget->setWhatsThis(QCoreApplication::translate("ChartDialog", "Complete list of metrics and instances which can be added to a new or current chart", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(whatsthis)
        availableMetricLineEdit->setWhatsThis(QCoreApplication::translate("ChartDialog", "Complete metric name of the currently selected metric in the Available Metrics list", nullptr));
#endif // QT_CONFIG(whatsthis)
        availableMetricLineEdit->setText(QString());
#if QT_CONFIG(tooltip)
        metricInfoButton->setToolTip(QCoreApplication::translate("ChartDialog", "Metric Info", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        metricInfoButton->setWhatsThis(QCoreApplication::translate("ChartDialog", "Display information about a metric selected from either of the Chart Plots or Available Metrics lists.  This includes metric descriptor, help text (if available), and value at the current time position.", nullptr));
#endif // QT_CONFIG(whatsthis)
        metricInfoButton->setText(QString());
#if QT_CONFIG(tooltip)
        metricSearchButton->setToolTip(QCoreApplication::translate("ChartDialog", "Metric Search", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        metricSearchButton->setWhatsThis(QCoreApplication::translate("ChartDialog", "Search for metrics using a regular expression on metric/instance names.", nullptr));
#endif // QT_CONFIG(whatsthis)
        metricSearchButton->setText(QString());
#if QT_CONFIG(tooltip)
        metricDeleteButton->setToolTip(QCoreApplication::translate("ChartDialog", "Remove plot from chart", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        metricDeleteButton->setWhatsThis(QCoreApplication::translate("ChartDialog", "Remove the current selected plot(s) from the Chart Plots list.  When OK is pressed, these plots will be removed from the current chart.", nullptr));
#endif // QT_CONFIG(whatsthis)
        metricDeleteButton->setText(QString());
#if QT_CONFIG(tooltip)
        metricAddButton->setToolTip(QCoreApplication::translate("ChartDialog", "Insert plot into chart", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        metricAddButton->setWhatsThis(QCoreApplication::translate("ChartDialog", "Add a new plot into the Chart Plots list.  When OK is pressed, this plot will be added to the new or current chart.", nullptr));
#endif // QT_CONFIG(whatsthis)
        metricAddButton->setText(QString());
#if QT_CONFIG(tooltip)
        sourceButton->setToolTip(QCoreApplication::translate("ChartDialog", "Add new host", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        sourceButton->setWhatsThis(QCoreApplication::translate("ChartDialog", "Add new metric source (host or archive) from which additional Available Metrics can be found for plotting.", nullptr));
#endif // QT_CONFIG(whatsthis)
        sourceButton->setText(QString());
        tabWidget->setTabText(tabWidget->indexOf(Widget3), QCoreApplication::translate("ChartDialog", "Metrics", nullptr));
#if QT_CONFIG(tooltip)
        revertColorButton->setToolTip(QCoreApplication::translate("ChartDialog", "Revert to previous color", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        revertColorButton->setStatusTip(QCoreApplication::translate("ChartDialog", "Change back to the previous plot color, as displayed in the frame below.", nullptr));
#endif // QT_CONFIG(statustip)
        revertColorButton->setText(QString());
#if QT_CONFIG(tooltip)
        applyColorButton->setToolTip(QCoreApplication::translate("ChartDialog", "Apply new plot color", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        applyColorButton->setWhatsThis(QCoreApplication::translate("ChartDialog", "Change the color of the currently selected plot in the Chart Plots list.", nullptr));
#endif // QT_CONFIG(whatsthis)
        applyColorButton->setText(QString());
#if QT_CONFIG(tooltip)
        saturationTextLabel->setToolTip(QCoreApplication::translate("ChartDialog", "Saturation", nullptr));
#endif // QT_CONFIG(tooltip)
        saturationTextLabel->setText(QCoreApplication::translate("ChartDialog", "S:", nullptr));
#if QT_CONFIG(tooltip)
        redTextLabel->setToolTip(QCoreApplication::translate("ChartDialog", "Red", nullptr));
#endif // QT_CONFIG(tooltip)
        redTextLabel->setText(QCoreApplication::translate("ChartDialog", "R:", nullptr));
#if QT_CONFIG(tooltip)
        rEd->setToolTip(QCoreApplication::translate("ChartDialog", "Red", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        bEd->setToolTip(QCoreApplication::translate("ChartDialog", "Blue", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        hueTextLabel->setToolTip(QCoreApplication::translate("ChartDialog", "Hue", nullptr));
#endif // QT_CONFIG(tooltip)
        hueTextLabel->setText(QCoreApplication::translate("ChartDialog", "H:", nullptr));
#if QT_CONFIG(tooltip)
        gEd->setToolTip(QCoreApplication::translate("ChartDialog", "Green", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        blueTextLabel->setToolTip(QCoreApplication::translate("ChartDialog", "Blue", nullptr));
#endif // QT_CONFIG(tooltip)
        blueTextLabel->setText(QCoreApplication::translate("ChartDialog", "B:", nullptr));
#if QT_CONFIG(tooltip)
        greenTextLabel->setToolTip(QCoreApplication::translate("ChartDialog", "Green", nullptr));
#endif // QT_CONFIG(tooltip)
        greenTextLabel->setText(QCoreApplication::translate("ChartDialog", "G:", nullptr));
#if QT_CONFIG(tooltip)
        sEd->setToolTip(QCoreApplication::translate("ChartDialog", "Saturation", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        hEd->setToolTip(QCoreApplication::translate("ChartDialog", "Hue", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        hEd->setWhatsThis(QString());
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(tooltip)
        vEd->setToolTip(QCoreApplication::translate("ChartDialog", "Value", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        valueTextLabel->setToolTip(QCoreApplication::translate("ChartDialog", "Value", nullptr));
#endif // QT_CONFIG(tooltip)
        valueTextLabel->setText(QCoreApplication::translate("ChartDialog", "V:", nullptr));
#if QT_CONFIG(tooltip)
        plotLabelLabel->setToolTip(QCoreApplication::translate("ChartDialog", "Label to use in chart legend for the currently selected Chart Plot.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        plotLabelLabel->setWhatsThis(QCoreApplication::translate("ChartDialog", "Label to use in chart legend for the currently selected Chart Plot.", nullptr));
#endif // QT_CONFIG(whatsthis)
        plotLabelLabel->setText(QCoreApplication::translate("ChartDialog", "Plot Label:", nullptr));
#if QT_CONFIG(tooltip)
        colorLineEdit->setToolTip(QCoreApplication::translate("ChartDialog", "Color specification string (e.g. RGB hex value)", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        colorLineEdit->setWhatsThis(QCoreApplication::translate("ChartDialog", "Directly edit the color specification string (e.g. RGB hex value) using this text box.  This color can then be used to set the color of the currently selected plot in the Chart Plots list using the left arrow button above.", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(tooltip)
        plotColorLabel->setToolTip(QCoreApplication::translate("ChartDialog", "Color specification string (e.g. RGB hex value)", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        plotColorLabel->setWhatsThis(QCoreApplication::translate("ChartDialog", "Directly edit the color specification string (e.g. RGB hex value) using this text box.  This color can then be used to set the color of the currently selected plot in the Chart Plots list using the left arrow button above.", nullptr));
#endif // QT_CONFIG(whatsthis)
        plotColorLabel->setText(QCoreApplication::translate("ChartDialog", "Plot Color:", nullptr));
#if QT_CONFIG(tooltip)
        plotLabelLineEdit->setToolTip(QCoreApplication::translate("ChartDialog", "Label to use in chart legend for the currently selected Chart Plot.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        plotLabelLineEdit->setWhatsThis(QCoreApplication::translate("ChartDialog", "Label to use in chart legend for the currently selected Chart Plot.", nullptr));
#endif // QT_CONFIG(whatsthis)
        tabWidget->setTabText(tabWidget->indexOf(TabPage), QCoreApplication::translate("ChartDialog", "Plots", nullptr));
        QTreeWidgetItem *___qtreewidgetitem1 = chartMetricsTreeWidget->headerItem();
        ___qtreewidgetitem1->setText(0, QCoreApplication::translate("ChartDialog", "Chart Plots", nullptr));
#if QT_CONFIG(whatsthis)
        chartMetricsTreeWidget->setWhatsThis(QCoreApplication::translate("ChartDialog", "List of plots for either a new chart or the currently selected chart", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(whatsthis)
        chartMetricLineEdit->setWhatsThis(QCoreApplication::translate("ChartDialog", "Displays the full metric name of the currently selected chart plot", nullptr));
#endif // QT_CONFIG(whatsthis)
        chartMetricLineEdit->setText(QString());
        buttonApply->setText(QCoreApplication::translate("ChartDialog", "&Apply", nullptr));
        buttonOk->setText(QCoreApplication::translate("ChartDialog", "&OK", nullptr));
#if QT_CONFIG(shortcut)
        buttonOk->setShortcut(QString());
#endif // QT_CONFIG(shortcut)
        buttonCancel->setText(QCoreApplication::translate("ChartDialog", "&Cancel", nullptr));
#if QT_CONFIG(shortcut)
        buttonCancel->setShortcut(QString());
#endif // QT_CONFIG(shortcut)
    } // retranslateUi

};

namespace Ui {
    class ChartDialog: public Ui_ChartDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_CHARTDIALOG_H
