/********************************************************************************
** Form generated from reading UI file 'openviewdialog.ui'
**
** Created by: Qt User Interface Compiler version 5.15.19
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_OPENVIEWDIALOG_H
#define UI_OPENVIEWDIALOG_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QListView>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QToolButton>

QT_BEGIN_NAMESPACE

class Ui_OpenViewDialog
{
public:
    QGridLayout *gridLayout;
    QGridLayout *gridLayout1;
    QSpacerItem *spacerItem;
    QPushButton *openPushButton;
    QComboBox *sourceComboBox;
    QPushButton *cancelPushButton;
    QSpacerItem *spacerItem1;
    QLabel *fileNameLabel;
    QLabel *sourceLabel;
    QPushButton *sourcePushButton;
    QSpacerItem *spacerItem2;
    QLineEdit *fileNameLineEdit;
    QSpacerItem *spacerItem3;
    QHBoxLayout *hboxLayout;
    QLabel *pathLabel;
    QComboBox *pathComboBox;
    QSpacerItem *spacerItem4;
    QToolButton *parentToolButton;
    QSpacerItem *spacerItem5;
    QToolButton *userToolButton;
    QToolButton *systemToolButton;
    QListView *dirListView;

    void setupUi(QDialog *OpenViewDialog)
    {
        if (OpenViewDialog->objectName().isEmpty())
            OpenViewDialog->setObjectName(QString::fromUtf8("OpenViewDialog"));
        OpenViewDialog->setWindowModality(Qt::NonModal);
        OpenViewDialog->resize(469, 283);
        const QIcon icon = QIcon(QString::fromUtf8(":/images/view.png"));
        OpenViewDialog->setWindowIcon(icon);
        OpenViewDialog->setSizeGripEnabled(true);
        gridLayout = new QGridLayout(OpenViewDialog);
#ifndef Q_OS_MAC
        gridLayout->setSpacing(6);
#endif
#ifndef Q_OS_MAC
        gridLayout->setContentsMargins(9, 9, 9, 9);
#endif
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        gridLayout1 = new QGridLayout();
        gridLayout1->setSpacing(0);
#ifndef Q_OS_MAC
        gridLayout1->setContentsMargins(0, 0, 0, 0);
#endif
        gridLayout1->setObjectName(QString::fromUtf8("gridLayout1"));
        spacerItem = new QSpacerItem(10, 20, QSizePolicy::Fixed, QSizePolicy::Minimum);

        gridLayout1->addItem(spacerItem, 1, 1, 1, 1);

        openPushButton = new QPushButton(OpenViewDialog);
        openPushButton->setObjectName(QString::fromUtf8("openPushButton"));
        openPushButton->setAutoDefault(false);

        gridLayout1->addWidget(openPushButton, 0, 4, 1, 1);

        sourceComboBox = new QComboBox(OpenViewDialog);
        sourceComboBox->setObjectName(QString::fromUtf8("sourceComboBox"));

        gridLayout1->addWidget(sourceComboBox, 1, 2, 1, 1);

        cancelPushButton = new QPushButton(OpenViewDialog);
        cancelPushButton->setObjectName(QString::fromUtf8("cancelPushButton"));

        gridLayout1->addWidget(cancelPushButton, 2, 4, 1, 1);

        spacerItem1 = new QSpacerItem(16, 29, QSizePolicy::Fixed, QSizePolicy::Minimum);

        gridLayout1->addItem(spacerItem1, 0, 3, 1, 1);

        fileNameLabel = new QLabel(OpenViewDialog);
        fileNameLabel->setObjectName(QString::fromUtf8("fileNameLabel"));
        fileNameLabel->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);

        gridLayout1->addWidget(fileNameLabel, 0, 0, 1, 1);

        sourceLabel = new QLabel(OpenViewDialog);
        sourceLabel->setObjectName(QString::fromUtf8("sourceLabel"));

        gridLayout1->addWidget(sourceLabel, 1, 0, 1, 1);

        sourcePushButton = new QPushButton(OpenViewDialog);
        sourcePushButton->setObjectName(QString::fromUtf8("sourcePushButton"));
        const QIcon icon1 = QIcon(QString::fromUtf8(":/images/computer.png"));
        sourcePushButton->setIcon(icon1);
        sourcePushButton->setIconSize(QSize(16, 16));

        gridLayout1->addWidget(sourcePushButton, 1, 4, 1, 1);

        spacerItem2 = new QSpacerItem(10, 20, QSizePolicy::Fixed, QSizePolicy::Minimum);

        gridLayout1->addItem(spacerItem2, 0, 1, 1, 1);

        fileNameLineEdit = new QLineEdit(OpenViewDialog);
        fileNameLineEdit->setObjectName(QString::fromUtf8("fileNameLineEdit"));

        gridLayout1->addWidget(fileNameLineEdit, 0, 2, 1, 1);

        spacerItem3 = new QSpacerItem(16, 32, QSizePolicy::Fixed, QSizePolicy::Minimum);

        gridLayout1->addItem(spacerItem3, 1, 3, 1, 1);


        gridLayout->addLayout(gridLayout1, 2, 0, 1, 1);

        hboxLayout = new QHBoxLayout();
        hboxLayout->setSpacing(0);
        hboxLayout->setContentsMargins(0, 0, 0, 0);
        hboxLayout->setObjectName(QString::fromUtf8("hboxLayout"));
        pathLabel = new QLabel(OpenViewDialog);
        pathLabel->setObjectName(QString::fromUtf8("pathLabel"));
        QSizePolicy sizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(pathLabel->sizePolicy().hasHeightForWidth());
        pathLabel->setSizePolicy(sizePolicy);
        pathLabel->setMinimumSize(QSize(60, 20));

        hboxLayout->addWidget(pathLabel);

        pathComboBox = new QComboBox(OpenViewDialog);
        pathComboBox->setObjectName(QString::fromUtf8("pathComboBox"));

        hboxLayout->addWidget(pathComboBox);

        spacerItem4 = new QSpacerItem(8, 26, QSizePolicy::Fixed, QSizePolicy::Minimum);

        hboxLayout->addItem(spacerItem4);

        parentToolButton = new QToolButton(OpenViewDialog);
        parentToolButton->setObjectName(QString::fromUtf8("parentToolButton"));
        parentToolButton->setMinimumSize(QSize(26, 26));
        parentToolButton->setMaximumSize(QSize(26, 26));
        const QIcon icon2 = QIcon(QString::fromUtf8(":/images/go-previous.png"));
        parentToolButton->setIcon(icon2);
        parentToolButton->setIconSize(QSize(16, 16));
        parentToolButton->setCheckable(false);

        hboxLayout->addWidget(parentToolButton);

        spacerItem5 = new QSpacerItem(8, 26, QSizePolicy::Fixed, QSizePolicy::Minimum);

        hboxLayout->addItem(spacerItem5);

        userToolButton = new QToolButton(OpenViewDialog);
        userToolButton->setObjectName(QString::fromUtf8("userToolButton"));
        userToolButton->setMinimumSize(QSize(26, 26));
        userToolButton->setMaximumSize(QSize(26, 26));
        const QIcon icon3 = QIcon(QString::fromUtf8(":/images/toolusers.png"));
        userToolButton->setIcon(icon3);
        userToolButton->setIconSize(QSize(16, 16));
        userToolButton->setCheckable(true);

        hboxLayout->addWidget(userToolButton);

        systemToolButton = new QToolButton(OpenViewDialog);
        systemToolButton->setObjectName(QString::fromUtf8("systemToolButton"));
        systemToolButton->setMinimumSize(QSize(26, 26));
        systemToolButton->setMaximumSize(QSize(26, 26));
        systemToolButton->setIcon(icon);
        systemToolButton->setIconSize(QSize(16, 16));
        systemToolButton->setCheckable(true);
        systemToolButton->setChecked(true);

        hboxLayout->addWidget(systemToolButton);


        gridLayout->addLayout(hboxLayout, 0, 0, 1, 1);

        dirListView = new QListView(OpenViewDialog);
        dirListView->setObjectName(QString::fromUtf8("dirListView"));
        QFont font;
        font.setPointSize(10);
        dirListView->setFont(font);
        dirListView->setSelectionMode(QAbstractItemView::ExtendedSelection);
        dirListView->setIconSize(QSize(20, 20));
        dirListView->setFlow(QListView::TopToBottom);
        dirListView->setProperty("isWrapping", QVariant(true));
        dirListView->setResizeMode(QListView::Adjust);
        dirListView->setGridSize(QSize(150, 20));
        dirListView->setUniformItemSizes(true);

        gridLayout->addWidget(dirListView, 1, 0, 1, 1);


        retranslateUi(OpenViewDialog);

        openPushButton->setDefault(false);
        cancelPushButton->setDefault(true);


        QMetaObject::connectSlotsByName(OpenViewDialog);
    } // setupUi

    void retranslateUi(QDialog *OpenViewDialog)
    {
        OpenViewDialog->setWindowTitle(QCoreApplication::translate("OpenViewDialog", "Open View", nullptr));
        openPushButton->setText(QCoreApplication::translate("OpenViewDialog", "Open", nullptr));
        cancelPushButton->setText(QCoreApplication::translate("OpenViewDialog", "Cancel", nullptr));
        fileNameLabel->setText(QCoreApplication::translate("OpenViewDialog", "File:", nullptr));
        sourceLabel->setText(QCoreApplication::translate("OpenViewDialog", "Source:", nullptr));
        sourcePushButton->setText(QCoreApplication::translate("OpenViewDialog", "...", nullptr));
        pathLabel->setText(QCoreApplication::translate("OpenViewDialog", "Path:", nullptr));
#if QT_CONFIG(tooltip)
        parentToolButton->setToolTip(QCoreApplication::translate("OpenViewDialog", "Parent", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        parentToolButton->setStatusTip(QString());
#endif // QT_CONFIG(statustip)
#if QT_CONFIG(whatsthis)
        parentToolButton->setWhatsThis(QCoreApplication::translate("OpenViewDialog", "Open the parent directory", nullptr));
#endif // QT_CONFIG(whatsthis)
        parentToolButton->setText(QString());
#if QT_CONFIG(tooltip)
        userToolButton->setToolTip(QCoreApplication::translate("OpenViewDialog", "User Views", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        userToolButton->setWhatsThis(QCoreApplication::translate("OpenViewDialog", "Display your personal saved Views.  A view is one or more charts.", nullptr));
#endif // QT_CONFIG(whatsthis)
        userToolButton->setText(QString());
#if QT_CONFIG(tooltip)
        systemToolButton->setToolTip(QCoreApplication::translate("OpenViewDialog", "System Views", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        systemToolButton->setWhatsThis(QCoreApplication::translate("OpenViewDialog", "Display preconfigured system Views.  A view is one or more charts.", nullptr));
#endif // QT_CONFIG(whatsthis)
        systemToolButton->setText(QString());
    } // retranslateUi

};

namespace Ui {
    class OpenViewDialog: public Ui_OpenViewDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_OPENVIEWDIALOG_H
