/********************************************************************************
** Form generated from reading UI file 'hostdialog.ui'
**
** Created by: Qt User Interface Compiler version 5.15.19
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_HOSTDIALOG_H
#define UI_HOSTDIALOG_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_HostDialog
{
public:
    QVBoxLayout *verticalLayout;
    QVBoxLayout *vboxLayout;
    QGridLayout *gridLayout;
    QLabel *hostLabel;
    QLineEdit *proxyLineEdit;
    QLineEdit *hostLineEdit;
    QCheckBox *proxyCheckBox;
    QCheckBox *containerCheckBox;
    QLineEdit *containerLineEdit;
    QGridLayout *encrypedGridLayout;
    QCheckBox *encryptedCheckBox;
    QCheckBox *authenticateCheckBox;
    QGridLayout *authenticateGridLayout;
    QLabel *usernameLabel;
    QLineEdit *usernameLineEdit;
    QLabel *passwordLabel;
    QLineEdit *passwordLineEdit;
    QLabel *realmLabel;
    QLineEdit *realmLineEdit;
    QSpacerItem *horizontalSpacer_2;
    QHBoxLayout *horizontalLayout;
    QPushButton *advancedPushButton;
    QSpacerItem *horizontalSpacer_3;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *HostDialog)
    {
        if (HostDialog->objectName().isEmpty())
            HostDialog->setObjectName(QString::fromUtf8("HostDialog"));
        HostDialog->setWindowModality(Qt::WindowModal);
        HostDialog->resize(426, 352);
        QSizePolicy sizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(HostDialog->sizePolicy().hasHeightForWidth());
        HostDialog->setSizePolicy(sizePolicy);
        HostDialog->setMinimumSize(QSize(300, 120));
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/images/computer.png"), QSize(), QIcon::Normal, QIcon::Off);
        HostDialog->setWindowIcon(icon);
        HostDialog->setSizeGripEnabled(false);
        HostDialog->setModal(true);
        verticalLayout = new QVBoxLayout(HostDialog);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        vboxLayout = new QVBoxLayout();
#ifndef Q_OS_MAC
        vboxLayout->setSpacing(6);
#endif
#ifndef Q_OS_MAC
        vboxLayout->setContentsMargins(0, 0, 0, 0);
#endif
        vboxLayout->setObjectName(QString::fromUtf8("vboxLayout"));
        gridLayout = new QGridLayout();
#ifndef Q_OS_MAC
        gridLayout->setSpacing(6);
#endif
#ifndef Q_OS_MAC
        gridLayout->setContentsMargins(0, 0, 0, 0);
#endif
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        hostLabel = new QLabel(HostDialog);
        hostLabel->setObjectName(QString::fromUtf8("hostLabel"));
        QFont font;
        font.setBold(true);
        font.setWeight(75);
        hostLabel->setFont(font);
        hostLabel->setFrameShadow(QFrame::Plain);
        hostLabel->setTextFormat(Qt::AutoText);
        hostLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        hostLabel->setWordWrap(false);

        gridLayout->addWidget(hostLabel, 0, 0, 1, 1);

        proxyLineEdit = new QLineEdit(HostDialog);
        proxyLineEdit->setObjectName(QString::fromUtf8("proxyLineEdit"));
        proxyLineEdit->setEnabled(false);
        proxyLineEdit->setMinimumSize(QSize(0, 27));

        gridLayout->addWidget(proxyLineEdit, 1, 1, 1, 1);

        hostLineEdit = new QLineEdit(HostDialog);
        hostLineEdit->setObjectName(QString::fromUtf8("hostLineEdit"));
        hostLineEdit->setMinimumSize(QSize(0, 27));

        gridLayout->addWidget(hostLineEdit, 0, 1, 1, 1);

        proxyCheckBox = new QCheckBox(HostDialog);
        proxyCheckBox->setObjectName(QString::fromUtf8("proxyCheckBox"));

        gridLayout->addWidget(proxyCheckBox, 1, 0, 1, 1);

        containerCheckBox = new QCheckBox(HostDialog);
        containerCheckBox->setObjectName(QString::fromUtf8("containerCheckBox"));

        gridLayout->addWidget(containerCheckBox, 2, 0, 1, 1);

        containerLineEdit = new QLineEdit(HostDialog);
        containerLineEdit->setObjectName(QString::fromUtf8("containerLineEdit"));
        containerLineEdit->setEnabled(false);

        gridLayout->addWidget(containerLineEdit, 2, 1, 1, 1);


        vboxLayout->addLayout(gridLayout);

        encrypedGridLayout = new QGridLayout();
        encrypedGridLayout->setObjectName(QString::fromUtf8("encrypedGridLayout"));
        encryptedCheckBox = new QCheckBox(HostDialog);
        encryptedCheckBox->setObjectName(QString::fromUtf8("encryptedCheckBox"));

        encrypedGridLayout->addWidget(encryptedCheckBox, 0, 0, 1, 1);


        vboxLayout->addLayout(encrypedGridLayout);

        authenticateCheckBox = new QCheckBox(HostDialog);
        authenticateCheckBox->setObjectName(QString::fromUtf8("authenticateCheckBox"));

        vboxLayout->addWidget(authenticateCheckBox);

        authenticateGridLayout = new QGridLayout();
        authenticateGridLayout->setObjectName(QString::fromUtf8("authenticateGridLayout"));
        usernameLabel = new QLabel(HostDialog);
        usernameLabel->setObjectName(QString::fromUtf8("usernameLabel"));
        usernameLabel->setEnabled(false);
        usernameLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        authenticateGridLayout->addWidget(usernameLabel, 0, 1, 1, 1);

        usernameLineEdit = new QLineEdit(HostDialog);
        usernameLineEdit->setObjectName(QString::fromUtf8("usernameLineEdit"));
        usernameLineEdit->setEnabled(false);

        authenticateGridLayout->addWidget(usernameLineEdit, 0, 2, 1, 1);

        passwordLabel = new QLabel(HostDialog);
        passwordLabel->setObjectName(QString::fromUtf8("passwordLabel"));
        passwordLabel->setEnabled(false);
        passwordLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        authenticateGridLayout->addWidget(passwordLabel, 1, 1, 1, 1);

        passwordLineEdit = new QLineEdit(HostDialog);
        passwordLineEdit->setObjectName(QString::fromUtf8("passwordLineEdit"));
        passwordLineEdit->setEnabled(false);
        passwordLineEdit->setEchoMode(QLineEdit::Password);

        authenticateGridLayout->addWidget(passwordLineEdit, 1, 2, 1, 1);

        realmLabel = new QLabel(HostDialog);
        realmLabel->setObjectName(QString::fromUtf8("realmLabel"));
        realmLabel->setEnabled(false);
        realmLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        authenticateGridLayout->addWidget(realmLabel, 2, 1, 1, 1);

        realmLineEdit = new QLineEdit(HostDialog);
        realmLineEdit->setObjectName(QString::fromUtf8("realmLineEdit"));
        realmLineEdit->setEnabled(false);

        authenticateGridLayout->addWidget(realmLineEdit, 2, 2, 1, 1);

        horizontalSpacer_2 = new QSpacerItem(10, 20, QSizePolicy::Fixed, QSizePolicy::Minimum);

        authenticateGridLayout->addItem(horizontalSpacer_2, 1, 0, 1, 1);


        vboxLayout->addLayout(authenticateGridLayout);


        verticalLayout->addLayout(vboxLayout);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        advancedPushButton = new QPushButton(HostDialog);
        advancedPushButton->setObjectName(QString::fromUtf8("advancedPushButton"));

        horizontalLayout->addWidget(advancedPushButton);

        horizontalSpacer_3 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout->addItem(horizontalSpacer_3);

        buttonBox = new QDialogButtonBox(HostDialog);
        buttonBox->setObjectName(QString::fromUtf8("buttonBox"));
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        horizontalLayout->addWidget(buttonBox);


        verticalLayout->addLayout(horizontalLayout);

#if QT_CONFIG(shortcut)
        hostLabel->setBuddy(hostLineEdit);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(hostLineEdit, proxyCheckBox);
        QWidget::setTabOrder(proxyCheckBox, proxyLineEdit);
        QWidget::setTabOrder(proxyLineEdit, containerCheckBox);
        QWidget::setTabOrder(containerCheckBox, encryptedCheckBox);
        QWidget::setTabOrder(encryptedCheckBox, authenticateCheckBox);
        QWidget::setTabOrder(authenticateCheckBox, usernameLineEdit);
        QWidget::setTabOrder(usernameLineEdit, passwordLineEdit);
        QWidget::setTabOrder(passwordLineEdit, realmLineEdit);
        QWidget::setTabOrder(realmLineEdit, advancedPushButton);
        QWidget::setTabOrder(advancedPushButton, buttonBox);

        retranslateUi(HostDialog);

        QMetaObject::connectSlotsByName(HostDialog);
    } // setupUi

    void retranslateUi(QDialog *HostDialog)
    {
        HostDialog->setWindowTitle(QCoreApplication::translate("HostDialog", "Add Host", nullptr));
        hostLabel->setText(QCoreApplication::translate("HostDialog", "Hostname:", nullptr));
        proxyCheckBox->setText(QCoreApplication::translate("HostDialog", "Proxy:", nullptr));
        containerCheckBox->setText(QCoreApplication::translate("HostDialog", "Container:", nullptr));
        encryptedCheckBox->setText(QCoreApplication::translate("HostDialog", "Encrypt", nullptr));
        authenticateCheckBox->setText(QCoreApplication::translate("HostDialog", "Authenticate:", nullptr));
        usernameLabel->setText(QCoreApplication::translate("HostDialog", "Username:", nullptr));
        passwordLabel->setText(QCoreApplication::translate("HostDialog", "Password:", nullptr));
        realmLabel->setText(QCoreApplication::translate("HostDialog", "Realm:", nullptr));
        advancedPushButton->setText(QCoreApplication::translate("HostDialog", "Advanced...", nullptr));
    } // retranslateUi

};

namespace Ui {
    class HostDialog: public Ui_HostDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_HOSTDIALOG_H
