/*
 * Performance Metrics Name Space Map
 * Built by pmgenmap.sh from the file
 * gpumetrics.map
 * on Thu Jul 30 14:21:13 UTC 2026
 *
 * Do not edit this file!
 */

const char *gpumetrics[] = {
/* per-process GPU values */
#define GPU_PROC_SAMPLES	0
	"nvidia.proc.all.samples",
#define GPU_PROC_MEMUSED	1
	"nvidia.proc.all.memused",
#define GPU_PROC_MEMACCUM	2
	"nvidia.proc.all.memaccum",
#define GPU_PROC_GPUBUSY	3
	"nvidia.proc.all.gpuactive",
#define GPU_PROC_MEMBUSY	4
	"nvidia.proc.all.memactive",
#define GPU_PROC_TIME	5
	"nvidia.proc.all.time",
#define GPU_PROC_RUNNING	6
	"nvidia.proc.all.running",
#define GPU_PROC_GPULIST	7
	"nvidia.proc.all.gpulist",
#define GPU_PROC_NGPUS	8
	"nvidia.proc.all.ngpus",

/* end marker - provides count */
#define GPU_NMETRICS	9
	".",

};

