/********************************************************************************
** Form generated from reading UI file 'qed_console.ui'
**
** Created by: Qt User Interface Compiler version 5.15.19
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_QED_CONSOLE_H
#define UI_QED_CONSOLE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTextEdit>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_QedConsole
{
public:
    QGridLayout *gridLayout;
    QVBoxLayout *vboxLayout;
    QTextEdit *text;
    QHBoxLayout *hboxLayout;
    QSpacerItem *spacerItem;
    QPushButton *buttonHide;

    void setupUi(QDialog *QedConsole)
    {
        if (QedConsole->objectName().isEmpty())
            QedConsole->setObjectName(QString::fromUtf8("QedConsole"));
        QedConsole->resize(493, 326);
        QedConsole->setSizeGripEnabled(true);
        gridLayout = new QGridLayout(QedConsole);
#ifndef Q_OS_MAC
        gridLayout->setSpacing(6);
#endif
#ifndef Q_OS_MAC
        gridLayout->setContentsMargins(9, 9, 9, 9);
#endif
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        vboxLayout = new QVBoxLayout();
#ifndef Q_OS_MAC
        vboxLayout->setSpacing(6);
#endif
#ifndef Q_OS_MAC
        vboxLayout->setContentsMargins(0, 0, 0, 0);
#endif
        vboxLayout->setObjectName(QString::fromUtf8("vboxLayout"));
        text = new QTextEdit(QedConsole);
        text->setObjectName(QString::fromUtf8("text"));
        QSizePolicy sizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(text->sizePolicy().hasHeightForWidth());
        text->setSizePolicy(sizePolicy);
        text->setAutoFormatting(QTextEdit::AutoNone);
        text->setReadOnly(true);

        vboxLayout->addWidget(text);

        hboxLayout = new QHBoxLayout();
#ifndef Q_OS_MAC
        hboxLayout->setSpacing(6);
#endif
        hboxLayout->setContentsMargins(0, 0, 0, 0);
        hboxLayout->setObjectName(QString::fromUtf8("hboxLayout"));
        spacerItem = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        hboxLayout->addItem(spacerItem);

        buttonHide = new QPushButton(QedConsole);
        buttonHide->setObjectName(QString::fromUtf8("buttonHide"));
        QSizePolicy sizePolicy1(QSizePolicy::Fixed, QSizePolicy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(buttonHide->sizePolicy().hasHeightForWidth());
        buttonHide->setSizePolicy(sizePolicy1);
        buttonHide->setMinimumSize(QSize(85, 30));
        buttonHide->setMaximumSize(QSize(85, 30));
        buttonHide->setAutoDefault(true);

        hboxLayout->addWidget(buttonHide);


        vboxLayout->addLayout(hboxLayout);


        gridLayout->addLayout(vboxLayout, 0, 0, 1, 1);


        retranslateUi(QedConsole);
        QObject::connect(buttonHide, SIGNAL(clicked()), QedConsole, SLOT(accept()));

        buttonHide->setDefault(true);


        QMetaObject::connectSlotsByName(QedConsole);
    } // setupUi

    void retranslateUi(QDialog *QedConsole)
    {
        QedConsole->setWindowTitle(QCoreApplication::translate("QedConsole", "PCP GUI Console", nullptr));
        buttonHide->setText(QCoreApplication::translate("QedConsole", "&Hide", nullptr));
#if QT_CONFIG(shortcut)
        buttonHide->setShortcut(QCoreApplication::translate("QedConsole", "Alt+H", nullptr));
#endif // QT_CONFIG(shortcut)
    } // retranslateUi

};

namespace Ui {
    class QedConsole: public Ui_QedConsole {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_QED_CONSOLE_H
