/*
 * Performance Metrics Name Space Map
 * Built by pmgenmap.sh from the file
 * acctmetrics.map
 * on Thu Jul 30 14:21:13 UTC 2026
 *
 * Do not edit this file!
 */

/*
 *
 * metric names and access macros for acctmetrics.h and modules.c
 *
 */
const char *acctmetrics[] = {
#define ACCT_GEN_EXCODE	0
	"acct.psinfo.exitcode",
#define ACCT_GEN_UID	1
	"acct.id.uid",
#define ACCT_GEN_GID	2
	"acct.id.gid",
#define ACCT_GEN_PPID	3
	"acct.psinfo.ppid",
#define ACCT_GEN_BTIME	4
	"acct.psinfo.btime",
#define ACCT_GEN_ETIME	5
	"acct.psinfo.etime",
#define ACCT_CPU_UTIME	6
	"acct.psinfo.utime",
#define ACCT_CPU_STIME	7
	"acct.psinfo.stime",
#define ACCT_DSK_RIO	8
	"acct.psinfo.rw",
#define ACCT_MEM_MINFLT	9
	"acct.psinfo.minflt",
#define ACCT_MEM_MAJFLT	10
	"acct.psinfo.majflt",
/* end marker - provides count */
#define ACCT_NMETRICS	11
	".",

};

