/********************************************************************************
** Form generated from reading UI file 'showboundsdialog.ui'
**
** Created by: Qt User Interface Compiler version 5.15.19
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SHOWBOUNDSDIALOG_H
#define UI_SHOWBOUNDSDIALOG_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include "console.h"
#include "qwt_slider.h"

QT_BEGIN_NAMESPACE

class Ui_ShowBounds
{
public:
    QGridLayout *gridLayout;
    QVBoxLayout *vboxLayout;
    QGridLayout *gridLayout1;
    QwtSlider *sliderEnd;
    QLabel *endLabel;
    QLabel *startLabel;
    QLineEdit *lineEditStart;
    QwtSlider *sliderStart;
    QLineEdit *lineEditEnd;
    QSpacerItem *spacerItem;
    QHBoxLayout *hboxLayout;
    QSpacerItem *spacerItem1;
    QPushButton *buttonOK;
    QPushButton *buttonCancel;

    void setupUi(QDialog *ShowBounds)
    {
        if (ShowBounds->objectName().isEmpty())
            ShowBounds->setObjectName(QString::fromUtf8("ShowBounds"));
        ShowBounds->resize(324, 181);
        ShowBounds->setAcceptDrops(false);
        const QIcon icon = QIcon(QString::fromUtf8(":/images/pmtime.png"));
        ShowBounds->setWindowIcon(icon);
        ShowBounds->setSizeGripEnabled(false);
        ShowBounds->setModal(false);
        gridLayout = new QGridLayout(ShowBounds);
        gridLayout->setSpacing(6);
        gridLayout->setContentsMargins(7, 7, 7, 7);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        vboxLayout = new QVBoxLayout();
        vboxLayout->setSpacing(4);
        vboxLayout->setContentsMargins(0, 0, 0, 0);
        vboxLayout->setObjectName(QString::fromUtf8("vboxLayout"));
        gridLayout1 = new QGridLayout();
        gridLayout1->setSpacing(6);
        gridLayout1->setContentsMargins(0, 0, 0, 0);
        gridLayout1->setObjectName(QString::fromUtf8("gridLayout1"));
        sliderEnd = new QwtSlider(ShowBounds);
        sliderEnd->setObjectName(QString::fromUtf8("sliderEnd"));
        sliderEnd->setOrientation(Qt::Horizontal);
        sliderEnd->setGroove(true);
        sliderEnd->setTrough(false);
        sliderEnd->setScalePosition(QwtSlider::NoScale);
        sliderEnd->setHandleSize(QSize(12, 16));

        gridLayout1->addWidget(sliderEnd, 3, 0, 1, 2);

        endLabel = new QLabel(ShowBounds);
        endLabel->setObjectName(QString::fromUtf8("endLabel"));
        endLabel->setWordWrap(false);

        gridLayout1->addWidget(endLabel, 2, 0, 1, 1);

        startLabel = new QLabel(ShowBounds);
        startLabel->setObjectName(QString::fromUtf8("startLabel"));
        startLabel->setWordWrap(false);

        gridLayout1->addWidget(startLabel, 0, 0, 1, 1);

        lineEditStart = new QLineEdit(ShowBounds);
        lineEditStart->setObjectName(QString::fromUtf8("lineEditStart"));

        gridLayout1->addWidget(lineEditStart, 0, 1, 1, 1);

        sliderStart = new QwtSlider(ShowBounds);
        sliderStart->setObjectName(QString::fromUtf8("sliderStart"));
        sliderStart->setOrientation(Qt::Horizontal);
        sliderStart->setGroove(true);
        sliderStart->setTrough(false);
        sliderStart->setScalePosition(QwtSlider::NoScale);
        sliderStart->setHandleSize(QSize(12, 16));

        gridLayout1->addWidget(sliderStart, 1, 0, 1, 2);

        lineEditEnd = new QLineEdit(ShowBounds);
        lineEditEnd->setObjectName(QString::fromUtf8("lineEditEnd"));

        gridLayout1->addWidget(lineEditEnd, 2, 1, 1, 1);


        vboxLayout->addLayout(gridLayout1);

        spacerItem = new QSpacerItem(20, 16, QSizePolicy::Minimum, QSizePolicy::Fixed);

        vboxLayout->addItem(spacerItem);

        hboxLayout = new QHBoxLayout();
        hboxLayout->setSpacing(6);
        hboxLayout->setContentsMargins(0, 0, 0, 0);
        hboxLayout->setObjectName(QString::fromUtf8("hboxLayout"));
        spacerItem1 = new QSpacerItem(20, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        hboxLayout->addItem(spacerItem1);

        buttonOK = new QPushButton(ShowBounds);
        buttonOK->setObjectName(QString::fromUtf8("buttonOK"));

        hboxLayout->addWidget(buttonOK);

        buttonCancel = new QPushButton(ShowBounds);
        buttonCancel->setObjectName(QString::fromUtf8("buttonCancel"));
        buttonCancel->setAutoDefault(true);

        hboxLayout->addWidget(buttonCancel);


        vboxLayout->addLayout(hboxLayout);


        gridLayout->addLayout(vboxLayout, 0, 0, 1, 1);


        retranslateUi(ShowBounds);
        QObject::connect(buttonOK, SIGNAL(clicked()), ShowBounds, SLOT(accept()));
        QObject::connect(buttonCancel, SIGNAL(clicked()), ShowBounds, SLOT(reject()));
        QObject::connect(sliderStart, SIGNAL(valueChanged(double)), ShowBounds, SLOT(changedStart(double)));
        QObject::connect(sliderEnd, SIGNAL(valueChanged(double)), ShowBounds, SLOT(changedEnd(double)));

        buttonCancel->setDefault(true);


        QMetaObject::connectSlotsByName(ShowBounds);
    } // setupUi

    void retranslateUi(QDialog *ShowBounds)
    {
        ShowBounds->setWindowTitle(QCoreApplication::translate("ShowBounds", "Archive Time Bounds", nullptr));
#if QT_CONFIG(whatsthis)
        ShowBounds->setWhatsThis(QCoreApplication::translate("ShowBounds", "The Archive Bounds dialog displays and allows modifications\n"
"to the start and end time of a (set of) PCP data archive(s).\n"
"This effectively restricts all pmtime clients (e.g. pmval, pmchart)\n"
"to a subset of the available performance data.", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(whatsthis)
        sliderEnd->setWhatsThis(QCoreApplication::translate("ShowBounds", "Modify the archive end time as seen by all<br>\n"
"pmtime clients (e.g. pmval, pmchart).<br><br>\n"
"This cannot be set before the start time of the<br>\n"
"archive with the earliest start time, nor beyond<br>\n"
"of the archive with the latest end time.<br>\n"
"It also cannot be less than the current start<br>\n"
"time.", nullptr));
#endif // QT_CONFIG(whatsthis)
        endLabel->setText(QCoreApplication::translate("ShowBounds", "End", nullptr));
        startLabel->setText(QCoreApplication::translate("ShowBounds", "Start", nullptr));
#if QT_CONFIG(whatsthis)
        lineEditStart->setWhatsThis(QCoreApplication::translate("ShowBounds", "Modify the archive start time as seen by all<br>\n"
"pmtime clients (e.g. pmval, pmchart).<br><br>\n"
"This cannot be set before the start time of the<br>\n"
"archive with the earliest start time, nor beyond<br>\n"
"of the archive with the latest end time.<br>\n"
"It also cannot be greater than the current end<br>\n"
"time.", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(whatsthis)
        sliderStart->setWhatsThis(QCoreApplication::translate("ShowBounds", "Modify the archive start time as seen by all<br>\n"
"pmtime clients (e.g. pmval, pmchart).<br><br>\n"
"This cannot be set before the start time of the<br>\n"
"archive with the earliest start time, nor beyond<br>\n"
"of the archive with the latest end time.<br>\n"
"It also cannot be greater than the current end<br>\n"
"time.", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(whatsthis)
        lineEditEnd->setWhatsThis(QCoreApplication::translate("ShowBounds", "Modify the archive end time as seen by all<br>\n"
"pmtime clients (e.g. pmval, pmchart).<br><br>\n"
"This cannot be set before the start time of the<br>\n"
"archive with the earliest start time, nor beyond<br>\n"
"of the archive with the latest end time.<br>\n"
"It also cannot be less than the current start<br>\n"
"time.", nullptr));
#endif // QT_CONFIG(whatsthis)
        buttonOK->setText(QCoreApplication::translate("ShowBounds", "&OK", nullptr));
        buttonCancel->setText(QCoreApplication::translate("ShowBounds", "&Cancel", nullptr));
#if QT_CONFIG(shortcut)
        buttonCancel->setShortcut(QString());
#endif // QT_CONFIG(shortcut)
    } // retranslateUi

};

namespace Ui {
    class ShowBounds: public Ui_ShowBounds {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_SHOWBOUNDSDIALOG_H
