/****************************************************************************
** Meta object code from reading C++ file 'timelord.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "timelord.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'timelord.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_TimeClient_t {
    QByteArrayData data[6];
    char stringdata0[63];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_TimeClient_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_TimeClient_t qt_meta_stringdata_TimeClient = {
    {
QT_MOC_LITERAL(0, 0, 10), // "TimeClient"
QT_MOC_LITERAL(1, 11, 10), // "endConnect"
QT_MOC_LITERAL(2, 22, 0), // ""
QT_MOC_LITERAL(3, 23, 11), // "TimeClient*"
QT_MOC_LITERAL(4, 35, 10), // "readClient"
QT_MOC_LITERAL(5, 46, 16) // "disconnectClient"

    },
    "TimeClient\0endConnect\0\0TimeClient*\0"
    "readClient\0disconnectClient"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_TimeClient[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       3,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   29,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       4,    0,   32,    2, 0x0a /* Public */,
       5,    0,   33,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    2,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void TimeClient::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<TimeClient *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->endConnect((*reinterpret_cast< TimeClient*(*)>(_a[1]))); break;
        case 1: _t->readClient(); break;
        case 2: _t->disconnectClient(); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< TimeClient* >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (TimeClient::*)(TimeClient * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TimeClient::endConnect)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject TimeClient::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_TimeClient.data,
    qt_meta_data_TimeClient,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *TimeClient::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TimeClient::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_TimeClient.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int TimeClient::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void TimeClient::endConnect(TimeClient * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
struct qt_meta_stringdata_TimeLord_t {
    QByteArrayData data[20];
    char stringdata0[160];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_TimeLord_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_TimeLord_t qt_meta_stringdata_TimeLord = {
    {
QT_MOC_LITERAL(0, 0, 8), // "TimeLord"
QT_MOC_LITERAL(1, 9, 14), // "lastClientExit"
QT_MOC_LITERAL(2, 24, 0), // ""
QT_MOC_LITERAL(3, 25, 4), // "quit"
QT_MOC_LITERAL(4, 30, 13), // "newConnection"
QT_MOC_LITERAL(5, 44, 10), // "endConnect"
QT_MOC_LITERAL(6, 55, 11), // "TimeClient*"
QT_MOC_LITERAL(7, 67, 6), // "client"
QT_MOC_LITERAL(8, 74, 9), // "timePulse"
QT_MOC_LITERAL(9, 84, 15), // "PmTime::Packet*"
QT_MOC_LITERAL(10, 100, 1), // "k"
QT_MOC_LITERAL(11, 102, 11), // "boundsPulse"
QT_MOC_LITERAL(12, 114, 12), // "vcrModePulse"
QT_MOC_LITERAL(13, 127, 4), // "drag"
QT_MOC_LITERAL(14, 132, 7), // "tzPulse"
QT_MOC_LITERAL(15, 140, 5), // "char*"
QT_MOC_LITERAL(16, 146, 1), // "t"
QT_MOC_LITERAL(17, 148, 4), // "tlen"
QT_MOC_LITERAL(18, 153, 1), // "l"
QT_MOC_LITERAL(19, 155, 4) // "llen"

    },
    "TimeLord\0lastClientExit\0\0quit\0"
    "newConnection\0endConnect\0TimeClient*\0"
    "client\0timePulse\0PmTime::Packet*\0k\0"
    "boundsPulse\0vcrModePulse\0drag\0tzPulse\0"
    "char*\0t\0tlen\0l\0llen"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_TimeLord[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       8,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   54,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       3,    0,   55,    2, 0x0a /* Public */,
       4,    0,   56,    2, 0x0a /* Public */,
       5,    1,   57,    2, 0x0a /* Public */,
       8,    1,   60,    2, 0x0a /* Public */,
      11,    1,   63,    2, 0x0a /* Public */,
      12,    2,   66,    2, 0x0a /* Public */,
      14,    5,   71,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 6,    7,
    QMetaType::Void, 0x80000000 | 9,   10,
    QMetaType::Void, 0x80000000 | 9,   10,
    QMetaType::Void, 0x80000000 | 9, QMetaType::Int,   10,   13,
    QMetaType::Void, 0x80000000 | 9, 0x80000000 | 15, QMetaType::Int, 0x80000000 | 15, QMetaType::Int,   10,   16,   17,   18,   19,

       0        // eod
};

void TimeLord::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<TimeLord *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->lastClientExit(); break;
        case 1: _t->quit(); break;
        case 2: _t->newConnection(); break;
        case 3: _t->endConnect((*reinterpret_cast< TimeClient*(*)>(_a[1]))); break;
        case 4: _t->timePulse((*reinterpret_cast< PmTime::Packet*(*)>(_a[1]))); break;
        case 5: _t->boundsPulse((*reinterpret_cast< PmTime::Packet*(*)>(_a[1]))); break;
        case 6: _t->vcrModePulse((*reinterpret_cast< PmTime::Packet*(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 7: _t->tzPulse((*reinterpret_cast< PmTime::Packet*(*)>(_a[1])),(*reinterpret_cast< char*(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3])),(*reinterpret_cast< char*(*)>(_a[4])),(*reinterpret_cast< int(*)>(_a[5]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< TimeClient* >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (TimeLord::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TimeLord::lastClientExit)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject TimeLord::staticMetaObject = { {
    QMetaObject::SuperData::link<QTcpServer::staticMetaObject>(),
    qt_meta_stringdata_TimeLord.data,
    qt_meta_data_TimeLord,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *TimeLord::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TimeLord::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_TimeLord.stringdata0))
        return static_cast<void*>(this);
    return QTcpServer::qt_metacast(_clname);
}

int TimeLord::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QTcpServer::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    return _id;
}

// SIGNAL 0
void TimeLord::lastClientExit()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
