/****************************************************************************
** Meta object code from reading C++ file 'timecontrol.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "timecontrol.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'timecontrol.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_TimeControl_t {
    QByteArrayData data[14];
    char stringdata0[262];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_TimeControl_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_TimeControl_t qt_meta_stringdata_TimeControl = {
    {
QT_MOC_LITERAL(0, 0, 11), // "TimeControl"
QT_MOC_LITERAL(1, 12, 19), // "showLiveTimeControl"
QT_MOC_LITERAL(2, 32, 0), // ""
QT_MOC_LITERAL(3, 33, 19), // "hideLiveTimeControl"
QT_MOC_LITERAL(4, 53, 22), // "showArchiveTimeControl"
QT_MOC_LITERAL(5, 76, 22), // "hideArchiveTimeControl"
QT_MOC_LITERAL(6, 99, 14), // "endTimeControl"
QT_MOC_LITERAL(7, 114, 18), // "readPortFromStdout"
QT_MOC_LITERAL(8, 133, 19), // "liveCloseConnection"
QT_MOC_LITERAL(9, 153, 19), // "liveSocketConnected"
QT_MOC_LITERAL(10, 173, 19), // "liveProtocolMessage"
QT_MOC_LITERAL(11, 193, 22), // "archiveCloseConnection"
QT_MOC_LITERAL(12, 216, 22), // "archiveSocketConnected"
QT_MOC_LITERAL(13, 239, 22) // "archiveProtocolMessage"

    },
    "TimeControl\0showLiveTimeControl\0\0"
    "hideLiveTimeControl\0showArchiveTimeControl\0"
    "hideArchiveTimeControl\0endTimeControl\0"
    "readPortFromStdout\0liveCloseConnection\0"
    "liveSocketConnected\0liveProtocolMessage\0"
    "archiveCloseConnection\0archiveSocketConnected\0"
    "archiveProtocolMessage"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_TimeControl[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      12,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   74,    2, 0x0a /* Public */,
       3,    0,   75,    2, 0x0a /* Public */,
       4,    0,   76,    2, 0x0a /* Public */,
       5,    0,   77,    2, 0x0a /* Public */,
       6,    0,   78,    2, 0x0a /* Public */,
       7,    0,   79,    2, 0x0a /* Public */,
       8,    0,   80,    2, 0x0a /* Public */,
       9,    0,   81,    2, 0x0a /* Public */,
      10,    0,   82,    2, 0x0a /* Public */,
      11,    0,   83,    2, 0x0a /* Public */,
      12,    0,   84,    2, 0x0a /* Public */,
      13,    0,   85,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void TimeControl::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<TimeControl *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->showLiveTimeControl(); break;
        case 1: _t->hideLiveTimeControl(); break;
        case 2: _t->showArchiveTimeControl(); break;
        case 3: _t->hideArchiveTimeControl(); break;
        case 4: _t->endTimeControl(); break;
        case 5: _t->readPortFromStdout(); break;
        case 6: _t->liveCloseConnection(); break;
        case 7: _t->liveSocketConnected(); break;
        case 8: _t->liveProtocolMessage(); break;
        case 9: _t->archiveCloseConnection(); break;
        case 10: _t->archiveSocketConnected(); break;
        case 11: _t->archiveProtocolMessage(); break;
        default: ;
        }
    }
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject TimeControl::staticMetaObject = { {
    QMetaObject::SuperData::link<QProcess::staticMetaObject>(),
    qt_meta_stringdata_TimeControl.data,
    qt_meta_data_TimeControl,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *TimeControl::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TimeControl::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_TimeControl.stringdata0))
        return static_cast<void*>(this);
    return QProcess::qt_metacast(_clname);
}

int TimeControl::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QProcess::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 12;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
