/********************************************************************************
** Form generated from reading UI file 'exportdialog.ui'
**
** Created by: Qt User Interface Compiler version 5.15.19
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_EXPORTDIALOG_H
#define UI_EXPORTDIALOG_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSlider>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_ExportDialog
{
public:
    QGridLayout *gridLayout;
    QHBoxLayout *hboxLayout;
    QVBoxLayout *vboxLayout;
    QHBoxLayout *hboxLayout1;
    QLabel *formatTextLabel;
    QComboBox *formatComboBox;
    QCheckBox *transparentCheckBox;
    QVBoxLayout *vboxLayout1;
    QRadioButton *selectedRadioButton;
    QRadioButton *allChartsRadioButton;
    QFrame *line;
    QGridLayout *gridLayout1;
    QSpinBox *heightSpinBox;
    QLabel *qualityLabel;
    QSpinBox *qualitySpinBox;
    QSlider *qualitySlider;
    QLabel *widthLabel;
    QLabel *heightLabel;
    QSpinBox *widthSpinBox;
    QHBoxLayout *hboxLayout2;
    QLabel *fileLabel;
    QLineEdit *fileLineEdit;
    QPushButton *filePushButton;
    QHBoxLayout *hboxLayout3;
    QSpacerItem *spacerItem;
    QPushButton *buttonOk;
    QPushButton *buttonCancel;

    void setupUi(QDialog *ExportDialog)
    {
        if (ExportDialog->objectName().isEmpty())
            ExportDialog->setObjectName(QString::fromUtf8("ExportDialog"));
        ExportDialog->setWindowModality(Qt::WindowModal);
        ExportDialog->resize(418, 218);
        QSizePolicy sizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(ExportDialog->sizePolicy().hasHeightForWidth());
        ExportDialog->setSizePolicy(sizePolicy);
        ExportDialog->setMinimumSize(QSize(418, 218));
        ExportDialog->setMaximumSize(QSize(418, 218));
        const QIcon icon = QIcon(QString::fromUtf8(":/images/document-export.png"));
        ExportDialog->setWindowIcon(icon);
        ExportDialog->setSizeGripEnabled(false);
        gridLayout = new QGridLayout(ExportDialog);
#ifndef Q_OS_MAC
        gridLayout->setSpacing(6);
#endif
#ifndef Q_OS_MAC
        gridLayout->setContentsMargins(9, 9, 9, 9);
#endif
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        hboxLayout = new QHBoxLayout();
        hboxLayout->setSpacing(20);
#ifndef Q_OS_MAC
        hboxLayout->setContentsMargins(0, 0, 0, 0);
#endif
        hboxLayout->setObjectName(QString::fromUtf8("hboxLayout"));
        vboxLayout = new QVBoxLayout();
#ifndef Q_OS_MAC
        vboxLayout->setSpacing(6);
#endif
#ifndef Q_OS_MAC
        vboxLayout->setContentsMargins(0, 0, 0, 0);
#endif
        vboxLayout->setObjectName(QString::fromUtf8("vboxLayout"));
        hboxLayout1 = new QHBoxLayout();
#ifndef Q_OS_MAC
        hboxLayout1->setSpacing(6);
#endif
#ifndef Q_OS_MAC
        hboxLayout1->setContentsMargins(0, 0, 0, 0);
#endif
        hboxLayout1->setObjectName(QString::fromUtf8("hboxLayout1"));
        formatTextLabel = new QLabel(ExportDialog);
        formatTextLabel->setObjectName(QString::fromUtf8("formatTextLabel"));
        formatTextLabel->setWordWrap(false);

        hboxLayout1->addWidget(formatTextLabel);

        formatComboBox = new QComboBox(ExportDialog);
        formatComboBox->setObjectName(QString::fromUtf8("formatComboBox"));
        formatComboBox->setMaxCount(32);
        formatComboBox->setDuplicatesEnabled(false);

        hboxLayout1->addWidget(formatComboBox);


        vboxLayout->addLayout(hboxLayout1);

        transparentCheckBox = new QCheckBox(ExportDialog);
        transparentCheckBox->setObjectName(QString::fromUtf8("transparentCheckBox"));
        transparentCheckBox->setChecked(true);

        vboxLayout->addWidget(transparentCheckBox);

        vboxLayout1 = new QVBoxLayout();
#ifndef Q_OS_MAC
        vboxLayout1->setSpacing(6);
#endif
        vboxLayout1->setContentsMargins(0, 0, 0, 0);
        vboxLayout1->setObjectName(QString::fromUtf8("vboxLayout1"));
        selectedRadioButton = new QRadioButton(ExportDialog);
        selectedRadioButton->setObjectName(QString::fromUtf8("selectedRadioButton"));
        selectedRadioButton->setChecked(false);

        vboxLayout1->addWidget(selectedRadioButton);

        allChartsRadioButton = new QRadioButton(ExportDialog);
        allChartsRadioButton->setObjectName(QString::fromUtf8("allChartsRadioButton"));
        QSizePolicy sizePolicy1(QSizePolicy::Minimum, QSizePolicy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(allChartsRadioButton->sizePolicy().hasHeightForWidth());
        allChartsRadioButton->setSizePolicy(sizePolicy1);
        allChartsRadioButton->setChecked(true);

        vboxLayout1->addWidget(allChartsRadioButton);


        vboxLayout->addLayout(vboxLayout1);


        hboxLayout->addLayout(vboxLayout);

        line = new QFrame(ExportDialog);
        line->setObjectName(QString::fromUtf8("line"));
        line->setFrameShape(QFrame::VLine);
        line->setFrameShadow(QFrame::Sunken);

        hboxLayout->addWidget(line);

        gridLayout1 = new QGridLayout();
        gridLayout1->setSpacing(0);
        gridLayout1->setContentsMargins(0, 0, 0, 0);
        gridLayout1->setObjectName(QString::fromUtf8("gridLayout1"));
        heightSpinBox = new QSpinBox(ExportDialog);
        heightSpinBox->setObjectName(QString::fromUtf8("heightSpinBox"));
        heightSpinBox->setMaximum(999999999);

        gridLayout1->addWidget(heightSpinBox, 1, 1, 1, 1);

        qualityLabel = new QLabel(ExportDialog);
        qualityLabel->setObjectName(QString::fromUtf8("qualityLabel"));
        qualityLabel->setWordWrap(false);

        gridLayout1->addWidget(qualityLabel, 2, 0, 1, 1);

        qualitySpinBox = new QSpinBox(ExportDialog);
        qualitySpinBox->setObjectName(QString::fromUtf8("qualitySpinBox"));
        qualitySpinBox->setMaximum(999999999);

        gridLayout1->addWidget(qualitySpinBox, 2, 1, 1, 1);

        qualitySlider = new QSlider(ExportDialog);
        qualitySlider->setObjectName(QString::fromUtf8("qualitySlider"));
        qualitySlider->setOrientation(Qt::Horizontal);

        gridLayout1->addWidget(qualitySlider, 3, 0, 1, 2);

        widthLabel = new QLabel(ExportDialog);
        widthLabel->setObjectName(QString::fromUtf8("widthLabel"));

        gridLayout1->addWidget(widthLabel, 0, 0, 1, 1);

        heightLabel = new QLabel(ExportDialog);
        heightLabel->setObjectName(QString::fromUtf8("heightLabel"));

        gridLayout1->addWidget(heightLabel, 1, 0, 1, 1);

        widthSpinBox = new QSpinBox(ExportDialog);
        widthSpinBox->setObjectName(QString::fromUtf8("widthSpinBox"));
        widthSpinBox->setMaximum(999999999);

        gridLayout1->addWidget(widthSpinBox, 0, 1, 1, 1);


        hboxLayout->addLayout(gridLayout1);


        gridLayout->addLayout(hboxLayout, 0, 0, 1, 1);

        hboxLayout2 = new QHBoxLayout();
#ifndef Q_OS_MAC
        hboxLayout2->setSpacing(6);
#endif
        hboxLayout2->setContentsMargins(0, 0, 0, 0);
        hboxLayout2->setObjectName(QString::fromUtf8("hboxLayout2"));
        fileLabel = new QLabel(ExportDialog);
        fileLabel->setObjectName(QString::fromUtf8("fileLabel"));
        fileLabel->setAlignment(Qt::AlignVCenter);
        fileLabel->setWordWrap(false);

        hboxLayout2->addWidget(fileLabel);

        fileLineEdit = new QLineEdit(ExportDialog);
        fileLineEdit->setObjectName(QString::fromUtf8("fileLineEdit"));
        fileLineEdit->setAlignment(Qt::AlignLeading);

        hboxLayout2->addWidget(fileLineEdit);

        filePushButton = new QPushButton(ExportDialog);
        filePushButton->setObjectName(QString::fromUtf8("filePushButton"));
        filePushButton->setIcon(icon);

        hboxLayout2->addWidget(filePushButton);


        gridLayout->addLayout(hboxLayout2, 1, 0, 1, 1);

        hboxLayout3 = new QHBoxLayout();
#ifndef Q_OS_MAC
        hboxLayout3->setSpacing(6);
#endif
        hboxLayout3->setContentsMargins(0, 0, 0, 0);
        hboxLayout3->setObjectName(QString::fromUtf8("hboxLayout3"));
        spacerItem = new QSpacerItem(214, 27, QSizePolicy::Expanding, QSizePolicy::Minimum);

        hboxLayout3->addItem(spacerItem);

        buttonOk = new QPushButton(ExportDialog);
        buttonOk->setObjectName(QString::fromUtf8("buttonOk"));
        buttonOk->setAutoDefault(true);

        hboxLayout3->addWidget(buttonOk);

        buttonCancel = new QPushButton(ExportDialog);
        buttonCancel->setObjectName(QString::fromUtf8("buttonCancel"));
        buttonCancel->setAutoDefault(true);

        hboxLayout3->addWidget(buttonCancel);


        gridLayout->addLayout(hboxLayout3, 2, 0, 1, 1);


        retranslateUi(ExportDialog);

        buttonOk->setDefault(true);


        QMetaObject::connectSlotsByName(ExportDialog);
    } // setupUi

    void retranslateUi(QDialog *ExportDialog)
    {
        ExportDialog->setWindowTitle(QCoreApplication::translate("ExportDialog", "Export", nullptr));
#if QT_CONFIG(tooltip)
        ExportDialog->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
        formatTextLabel->setText(QCoreApplication::translate("ExportDialog", "Image Format", nullptr));
        transparentCheckBox->setText(QCoreApplication::translate("ExportDialog", "Transparent Background", nullptr));
        selectedRadioButton->setText(QCoreApplication::translate("ExportDialog", "Selected Chart Only", nullptr));
        allChartsRadioButton->setText(QCoreApplication::translate("ExportDialog", "All Charts in Tab", nullptr));
        heightSpinBox->setSuffix(QCoreApplication::translate("ExportDialog", " px", nullptr));
        qualityLabel->setText(QCoreApplication::translate("ExportDialog", "Quality", nullptr));
        qualitySpinBox->setSuffix(QCoreApplication::translate("ExportDialog", "%", nullptr));
        widthLabel->setText(QCoreApplication::translate("ExportDialog", "Width", nullptr));
        heightLabel->setText(QCoreApplication::translate("ExportDialog", "Height", nullptr));
        widthSpinBox->setSuffix(QCoreApplication::translate("ExportDialog", " px", nullptr));
        fileLabel->setText(QCoreApplication::translate("ExportDialog", "File", nullptr));
        fileLineEdit->setText(QString());
        filePushButton->setText(QCoreApplication::translate("ExportDialog", "...", nullptr));
        buttonOk->setText(QCoreApplication::translate("ExportDialog", "&OK", nullptr));
#if QT_CONFIG(shortcut)
        buttonOk->setShortcut(QString());
#endif // QT_CONFIG(shortcut)
        buttonCancel->setText(QCoreApplication::translate("ExportDialog", "&Cancel", nullptr));
#if QT_CONFIG(shortcut)
        buttonCancel->setShortcut(QString());
#endif // QT_CONFIG(shortcut)
    } // retranslateUi

};

namespace Ui {
    class ExportDialog: public Ui_ExportDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_EXPORTDIALOG_H
