/****************************************************************************
** Meta object code from reading C++ file 'settingsdialog.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "settingsdialog.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'settingsdialog.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_SettingsDialog_t {
    QByteArrayData data[57];
    char stringdata0[1376];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_SettingsDialog_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_SettingsDialog_t qt_meta_stringdata_SettingsDialog = {
    {
QT_MOC_LITERAL(0, 0, 14), // "SettingsDialog"
QT_MOC_LITERAL(1, 15, 26), // "settingsTab_currentChanged"
QT_MOC_LITERAL(2, 42, 0), // ""
QT_MOC_LITERAL(3, 43, 5), // "index"
QT_MOC_LITERAL(4, 49, 34), // "chartDeltaLineEdit_editingFin..."
QT_MOC_LITERAL(5, 84, 35), // "loggerDeltaLineEdit_editingFi..."
QT_MOC_LITERAL(6, 120, 33), // "chartDeltaUnitsComboBox_activ..."
QT_MOC_LITERAL(7, 154, 5), // "value"
QT_MOC_LITERAL(8, 160, 34), // "loggerDeltaUnitsComboBox_acti..."
QT_MOC_LITERAL(9, 195, 20), // "visible_valueChanged"
QT_MOC_LITERAL(10, 216, 19), // "sample_valueChanged"
QT_MOC_LITERAL(11, 236, 31), // "selectedHighlightButton_clicked"
QT_MOC_LITERAL(12, 268, 31), // "defaultBackgroundButton_clicked"
QT_MOC_LITERAL(13, 300, 18), // "colorButtonClicked"
QT_MOC_LITERAL(14, 319, 26), // "removeSchemeButton_clicked"
QT_MOC_LITERAL(15, 346, 26), // "updateSchemeButton_clicked"
QT_MOC_LITERAL(16, 373, 34), // "schemeComboBox_currentIndexCh..."
QT_MOC_LITERAL(17, 408, 30), // "familyLineEdit_editingFinished"
QT_MOC_LITERAL(18, 439, 28), // "familyListWidget_itemClicked"
QT_MOC_LITERAL(19, 468, 16), // "QListWidgetItem*"
QT_MOC_LITERAL(20, 485, 29), // "styleLineEdit_editingFinished"
QT_MOC_LITERAL(21, 515, 27), // "styleListWidget_itemClicked"
QT_MOC_LITERAL(22, 543, 28), // "sizeLineEdit_editingFinished"
QT_MOC_LITERAL(23, 572, 26), // "sizeListWidget_itemClicked"
QT_MOC_LITERAL(24, 599, 23), // "resetFontButton_clicked"
QT_MOC_LITERAL(25, 623, 23), // "applyFontButton_clicked"
QT_MOC_LITERAL(26, 647, 18), // "hostButton_clicked"
QT_MOC_LITERAL(27, 666, 41), // "savedHostsListWidget_itemSele..."
QT_MOC_LITERAL(28, 708, 24), // "removeHostButton_clicked"
QT_MOC_LITERAL(29, 733, 21), // "addHostButton_clicked"
QT_MOC_LITERAL(30, 755, 30), // "startupToolbarCheckBox_clicked"
QT_MOC_LITERAL(31, 786, 29), // "nativeToolbarCheckBox_clicked"
QT_MOC_LITERAL(32, 816, 40), // "toolbarAreasComboBox_currentI..."
QT_MOC_LITERAL(33, 857, 28), // "actionListWidget_itemClicked"
QT_MOC_LITERAL(34, 886, 14), // "languageChange"
QT_MOC_LITERAL(35, 901, 20), // "colorButton1_clicked"
QT_MOC_LITERAL(36, 922, 20), // "colorButton2_clicked"
QT_MOC_LITERAL(37, 943, 20), // "colorButton3_clicked"
QT_MOC_LITERAL(38, 964, 20), // "colorButton4_clicked"
QT_MOC_LITERAL(39, 985, 20), // "colorButton5_clicked"
QT_MOC_LITERAL(40, 1006, 20), // "colorButton6_clicked"
QT_MOC_LITERAL(41, 1027, 20), // "colorButton7_clicked"
QT_MOC_LITERAL(42, 1048, 20), // "colorButton8_clicked"
QT_MOC_LITERAL(43, 1069, 20), // "colorButton9_clicked"
QT_MOC_LITERAL(44, 1090, 21), // "colorButton10_clicked"
QT_MOC_LITERAL(45, 1112, 21), // "colorButton11_clicked"
QT_MOC_LITERAL(46, 1134, 21), // "colorButton12_clicked"
QT_MOC_LITERAL(47, 1156, 21), // "colorButton13_clicked"
QT_MOC_LITERAL(48, 1178, 21), // "colorButton14_clicked"
QT_MOC_LITERAL(49, 1200, 21), // "colorButton15_clicked"
QT_MOC_LITERAL(50, 1222, 21), // "colorButton16_clicked"
QT_MOC_LITERAL(51, 1244, 21), // "colorButton17_clicked"
QT_MOC_LITERAL(52, 1266, 21), // "colorButton18_clicked"
QT_MOC_LITERAL(53, 1288, 21), // "colorButton19_clicked"
QT_MOC_LITERAL(54, 1310, 21), // "colorButton20_clicked"
QT_MOC_LITERAL(55, 1332, 21), // "colorButton21_clicked"
QT_MOC_LITERAL(56, 1354, 21) // "colorButton22_clicked"

    },
    "SettingsDialog\0settingsTab_currentChanged\0"
    "\0index\0chartDeltaLineEdit_editingFinished\0"
    "loggerDeltaLineEdit_editingFinished\0"
    "chartDeltaUnitsComboBox_activated\0"
    "value\0loggerDeltaUnitsComboBox_activated\0"
    "visible_valueChanged\0sample_valueChanged\0"
    "selectedHighlightButton_clicked\0"
    "defaultBackgroundButton_clicked\0"
    "colorButtonClicked\0removeSchemeButton_clicked\0"
    "updateSchemeButton_clicked\0"
    "schemeComboBox_currentIndexChanged\0"
    "familyLineEdit_editingFinished\0"
    "familyListWidget_itemClicked\0"
    "QListWidgetItem*\0styleLineEdit_editingFinished\0"
    "styleListWidget_itemClicked\0"
    "sizeLineEdit_editingFinished\0"
    "sizeListWidget_itemClicked\0"
    "resetFontButton_clicked\0applyFontButton_clicked\0"
    "hostButton_clicked\0"
    "savedHostsListWidget_itemSelectionChanged\0"
    "removeHostButton_clicked\0addHostButton_clicked\0"
    "startupToolbarCheckBox_clicked\0"
    "nativeToolbarCheckBox_clicked\0"
    "toolbarAreasComboBox_currentIndexChanged\0"
    "actionListWidget_itemClicked\0"
    "languageChange\0colorButton1_clicked\0"
    "colorButton2_clicked\0colorButton3_clicked\0"
    "colorButton4_clicked\0colorButton5_clicked\0"
    "colorButton6_clicked\0colorButton7_clicked\0"
    "colorButton8_clicked\0colorButton9_clicked\0"
    "colorButton10_clicked\0colorButton11_clicked\0"
    "colorButton12_clicked\0colorButton13_clicked\0"
    "colorButton14_clicked\0colorButton15_clicked\0"
    "colorButton16_clicked\0colorButton17_clicked\0"
    "colorButton18_clicked\0colorButton19_clicked\0"
    "colorButton20_clicked\0colorButton21_clicked\0"
    "colorButton22_clicked"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_SettingsDialog[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      52,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,  274,    2, 0x0a /* Public */,
       4,    0,  277,    2, 0x0a /* Public */,
       5,    0,  278,    2, 0x0a /* Public */,
       6,    1,  279,    2, 0x0a /* Public */,
       8,    1,  282,    2, 0x0a /* Public */,
       9,    1,  285,    2, 0x0a /* Public */,
      10,    1,  288,    2, 0x0a /* Public */,
      11,    0,  291,    2, 0x0a /* Public */,
      12,    0,  292,    2, 0x0a /* Public */,
      13,    1,  293,    2, 0x0a /* Public */,
      14,    0,  296,    2, 0x0a /* Public */,
      15,    0,  297,    2, 0x0a /* Public */,
      16,    1,  298,    2, 0x0a /* Public */,
      17,    0,  301,    2, 0x0a /* Public */,
      18,    1,  302,    2, 0x0a /* Public */,
      20,    0,  305,    2, 0x0a /* Public */,
      21,    1,  306,    2, 0x0a /* Public */,
      22,    0,  309,    2, 0x0a /* Public */,
      23,    1,  310,    2, 0x0a /* Public */,
      24,    0,  313,    2, 0x0a /* Public */,
      25,    0,  314,    2, 0x0a /* Public */,
      26,    0,  315,    2, 0x0a /* Public */,
      27,    0,  316,    2, 0x0a /* Public */,
      28,    0,  317,    2, 0x0a /* Public */,
      29,    0,  318,    2, 0x0a /* Public */,
      30,    0,  319,    2, 0x0a /* Public */,
      31,    0,  320,    2, 0x0a /* Public */,
      32,    1,  321,    2, 0x0a /* Public */,
      33,    1,  324,    2, 0x0a /* Public */,
      34,    0,  327,    2, 0x09 /* Protected */,
      35,    0,  328,    2, 0x09 /* Protected */,
      36,    0,  329,    2, 0x09 /* Protected */,
      37,    0,  330,    2, 0x09 /* Protected */,
      38,    0,  331,    2, 0x09 /* Protected */,
      39,    0,  332,    2, 0x09 /* Protected */,
      40,    0,  333,    2, 0x09 /* Protected */,
      41,    0,  334,    2, 0x09 /* Protected */,
      42,    0,  335,    2, 0x09 /* Protected */,
      43,    0,  336,    2, 0x09 /* Protected */,
      44,    0,  337,    2, 0x09 /* Protected */,
      45,    0,  338,    2, 0x09 /* Protected */,
      46,    0,  339,    2, 0x09 /* Protected */,
      47,    0,  340,    2, 0x09 /* Protected */,
      48,    0,  341,    2, 0x09 /* Protected */,
      49,    0,  342,    2, 0x09 /* Protected */,
      50,    0,  343,    2, 0x09 /* Protected */,
      51,    0,  344,    2, 0x09 /* Protected */,
      52,    0,  345,    2, 0x09 /* Protected */,
      53,    0,  346,    2, 0x09 /* Protected */,
      54,    0,  347,    2, 0x09 /* Protected */,
      55,    0,  348,    2, 0x09 /* Protected */,
      56,    0,  349,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    7,
    QMetaType::Void, QMetaType::Int,    7,
    QMetaType::Void, QMetaType::Int,    7,
    QMetaType::Void, QMetaType::Int,    7,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 19,    2,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 19,    2,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 19,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, 0x80000000 | 19,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void SettingsDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<SettingsDialog *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->settingsTab_currentChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 1: _t->chartDeltaLineEdit_editingFinished(); break;
        case 2: _t->loggerDeltaLineEdit_editingFinished(); break;
        case 3: _t->chartDeltaUnitsComboBox_activated((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 4: _t->loggerDeltaUnitsComboBox_activated((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 5: _t->visible_valueChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 6: _t->sample_valueChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 7: _t->selectedHighlightButton_clicked(); break;
        case 8: _t->defaultBackgroundButton_clicked(); break;
        case 9: _t->colorButtonClicked((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 10: _t->removeSchemeButton_clicked(); break;
        case 11: _t->updateSchemeButton_clicked(); break;
        case 12: _t->schemeComboBox_currentIndexChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 13: _t->familyLineEdit_editingFinished(); break;
        case 14: _t->familyListWidget_itemClicked((*reinterpret_cast< QListWidgetItem*(*)>(_a[1]))); break;
        case 15: _t->styleLineEdit_editingFinished(); break;
        case 16: _t->styleListWidget_itemClicked((*reinterpret_cast< QListWidgetItem*(*)>(_a[1]))); break;
        case 17: _t->sizeLineEdit_editingFinished(); break;
        case 18: _t->sizeListWidget_itemClicked((*reinterpret_cast< QListWidgetItem*(*)>(_a[1]))); break;
        case 19: _t->resetFontButton_clicked(); break;
        case 20: _t->applyFontButton_clicked(); break;
        case 21: _t->hostButton_clicked(); break;
        case 22: _t->savedHostsListWidget_itemSelectionChanged(); break;
        case 23: _t->removeHostButton_clicked(); break;
        case 24: _t->addHostButton_clicked(); break;
        case 25: _t->startupToolbarCheckBox_clicked(); break;
        case 26: _t->nativeToolbarCheckBox_clicked(); break;
        case 27: _t->toolbarAreasComboBox_currentIndexChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 28: _t->actionListWidget_itemClicked((*reinterpret_cast< QListWidgetItem*(*)>(_a[1]))); break;
        case 29: _t->languageChange(); break;
        case 30: _t->colorButton1_clicked(); break;
        case 31: _t->colorButton2_clicked(); break;
        case 32: _t->colorButton3_clicked(); break;
        case 33: _t->colorButton4_clicked(); break;
        case 34: _t->colorButton5_clicked(); break;
        case 35: _t->colorButton6_clicked(); break;
        case 36: _t->colorButton7_clicked(); break;
        case 37: _t->colorButton8_clicked(); break;
        case 38: _t->colorButton9_clicked(); break;
        case 39: _t->colorButton10_clicked(); break;
        case 40: _t->colorButton11_clicked(); break;
        case 41: _t->colorButton12_clicked(); break;
        case 42: _t->colorButton13_clicked(); break;
        case 43: _t->colorButton14_clicked(); break;
        case 44: _t->colorButton15_clicked(); break;
        case 45: _t->colorButton16_clicked(); break;
        case 46: _t->colorButton17_clicked(); break;
        case 47: _t->colorButton18_clicked(); break;
        case 48: _t->colorButton19_clicked(); break;
        case 49: _t->colorButton20_clicked(); break;
        case 50: _t->colorButton21_clicked(); break;
        case 51: _t->colorButton22_clicked(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject SettingsDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_meta_stringdata_SettingsDialog.data,
    qt_meta_data_SettingsDialog,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *SettingsDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SettingsDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_SettingsDialog.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui::SettingsDialog"))
        return static_cast< Ui::SettingsDialog*>(this);
    return QDialog::qt_metacast(_clname);
}

int SettingsDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 52)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 52;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 52)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 52;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
